# generated by datamodel-codegen:
#   filename:  data.json
#   timestamp: 2021-01-27T04:52:30+00:00

from __future__ import annotations

from typing import Any, Optional

from pydantic import BaseModel


class ServerIDModel(BaseModel):
    identifier: Optional[Any] = None






import json
from typing import Optional, List

from pydantic import BaseModel


class DiscordWebhookEmbedImage(BaseModel):
    """
    An image from a Discord-style webhook embed
    """
    url: str


class DiscordWebhookEmbedField(BaseModel):
    """
    A field from a Discord-style webhook embed
    """
    name: str
    value: str
    inline: Optional[bool] = False


class DiscordWebhookEmbed(BaseModel):
    """
    An embed from a Discord-style webhook
    """
    title: Optional[str] = None
    timestamp: Optional[str] = None
    thumbnail: Optional[DiscordWebhookEmbedImage] = None
    image: Optional[DiscordWebhookEmbedImage] = None
    color: Optional[int] = None
    description: Optional[str] = None
    fields: Optional[List[DiscordWebhookEmbedField]] = []


class DiscordWebhookData(BaseModel):
    """
    Data from a Discord-style webhook
    """
    content: str
    username: Optional[str] = None
    avatar_url: Optional[str] = None
    tts: Optional[bool] = False
    embeds: Optional[List[DiscordWebhookEmbed]] = []

    @classmethod
    def from_flask_request_body(cls, body: dict) -> "DiscordWebhookData":
        """
        Ingest a Discord-style webhook from a Flask request body
        :param body: The body of the Flask request
        :return: Data from the Discord-style webhook
        """
        return cls(**body)

    @classmethod
    def from_fastapi_request_body(cls, body: dict) -> "DiscordWebhookData":
        """
        Ingest a Discord-style webhook from a FastAPI request body
        :param body: The body of the FastAPI request
        :return: Data from the Discord-style webhook
        """
        return cls(**body)


class DiscordWebhookAttachment(BaseModel):
    """
    An attachment from a Discord-style webhook
    """
    filename: str
    file_type: str
    content: bytes

    @classmethod
    def from_flask_request_files(cls, files: dict) -> List["DiscordWebhookAttachment"]:
        """
        Ingest a list of files from a Discord webhook Flask request
        :param files: The files from the Flask request
        :return: A list of Discord webhook attachments
        """
        # Will probably only have one key, 'files[0]'
        # ref: https://github.com/Tautulli/Tautulli/blob/d019efcf911b4806618761c2da48bab7d04031ec/plexpy/notifiers.py#L1170
        for key, value in files.items():
            content = value.read()
            yield cls(filename=value.filename, file_type=value.content_type, content=content)


# ref: https://github.com/Tautulli/Tautulli/blob/d019efcf911b4806618761c2da48bab7d04031ec/plexpy/notifiers.py#L1148
class DiscordWebhookIngestor(BaseModel):
    """
    An ingestor for Discord-style webhooks from Tautulli
    """
    data: DiscordWebhookData
    attachments: Optional[List[DiscordWebhookAttachment]]

    @classmethod
    def from_flask_request(cls, request) -> "DiscordWebhookIngestor":
        """
        Ingest a Discord-style webhook from a Flask request

        :param request: The incoming Flask request
        :return: The processed Discord-style webhook
        """
        try:
            body = request.get_json()
        except Exception:
            # JSON data is stored in the form field 'payload_json' if files are present
            # ref: https://github.com/Tautulli/Tautulli/blob/d019efcf911b4806618761c2da48bab7d04031ec/plexpy/notifiers.py#L1225
            body = json.loads(request.form.get('payload_json', '{}'))
        files = request.files

        data = DiscordWebhookData.from_flask_request_body(body=body)
        attachments = list(DiscordWebhookAttachment.from_flask_request_files(files=files))

        return cls(data=data, attachments=attachments)



# generated by datamodel-codegen:
#   filename:  data.json
#   timestamp: 2021-01-27T04:54:28+00:00

from __future__ import annotations

from typing import Any, Optional

from pydantic import BaseModel


class StreamDataModel(BaseModel):
    bitrate: Optional[int] = None
    video_full_resolution: Optional[str] = None
    optimized_version: Optional[str] = None
    optimized_version_profile: Optional[str] = None
    optimized_version_title: Optional[str] = None
    synced_version: Optional[str] = None
    synced_version_profile: Optional[str] = None
    container: Optional[str] = None
    video_codec: Optional[str] = None
    video_bitrate: Optional[int] = None
    video_width: Optional[int] = None
    video_height: Optional[int] = None
    video_framerate: Optional[str] = None
    video_dynamic_range: Optional[str] = None
    aspect_ratio: Optional[str] = None
    audio_codec: Optional[str] = None
    audio_bitrate: Optional[int] = None
    audio_channels: Optional[int] = None
    subtitle_codec: Optional[str] = None
    stream_bitrate: Optional[int] = None
    stream_video_full_resolution: Optional[str] = None
    quality_profile: Optional[str] = None
    stream_container_decision: Optional[str] = None
    stream_container: Optional[str] = None
    stream_video_decision: Optional[str] = None
    stream_video_codec: Optional[str] = None
    stream_video_bitrate: Optional[int] = None
    stream_video_width: Optional[int] = None
    stream_video_height: Optional[int] = None
    stream_video_framerate: Optional[str] = None
    stream_video_dynamic_range: Optional[str] = None
    stream_audio_decision: Optional[str] = None
    stream_audio_codec: Optional[str] = None
    stream_audio_bitrate: Optional[int] = None
    stream_audio_channels: Optional[int] = None
    subtitles: Optional[str] = None
    stream_subtitle_decision: Optional[str] = None
    stream_subtitle_codec: Optional[str] = None
    transcode_hw_decoding: Optional[str] = None
    transcode_hw_encoding: Optional[str] = None
    video_decision: Optional[str] = None
    audio_decision: Optional[str] = None
    media_type: Optional[str] = None
    title: Optional[str] = None
    grandparent_title: Optional[str] = None
    original_title: Optional[str] = None
    current_session: Optional[int] = None
    pre_tautulli: Optional[str] = None






from urllib.parse import quote_plus


def _human_bitrate(number, denominator: int = 1, letter: str = "", d: int = 1):
    if d <= 0:
        return f'{int(number / denominator):d} {letter}bps'
    else:
        return f'{float(number / denominator):.{d}f} {letter}bps'


# noinspection PyPep8Naming
def to_human_bitrate(kilobytes, d: int = 1) -> str:
    # Return the given kilobytes as a human friendly bps, Kbps, Mbps, Gbps, or Tbps string

    KB = float(1024)
    MB = float(KB ** 2)  # 1,048,576
    GB = float(KB ** 3)  # 1,073,741,824
    TB = float(KB ** 4)  # 1,099,511,627,776

    denominator = 1
    letter = ""
    if kilobytes < KB:
        pass
    elif KB <= kilobytes < MB:
        denominator = KB
        letter = "k"
    elif MB <= kilobytes < GB:
        denominator = MB
        letter = "M"
    elif GB <= kilobytes < TB:
        denominator = GB
        letter = "G"
    else:
        denominator = TB
        letter = "T"

    return _human_bitrate(kilobytes, denominator=denominator, letter=letter, d=d)


def redact(full_string: str, to_redact: str, redaction: str = "[REDACTED]") -> str:
    """
    Redact a string from a full string

    :param full_string: Full string to redact from
    :type full_string: str
    :param to_redact: String to redact
    :type to_redact: str
    :param redaction: String to replace the redacted string with
    :type redaction: str
    :returns: Redacted string
    :rtype: str
    """
    return full_string.replace(to_redact, redaction)


def url_encode(string: str) -> str:
    """
    URL encode a string

    :param string: String to encode
    :type string: str
    :returns: URL encoded string
    :rtype: str
    """
    return quote_plus(string)



import warnings
from datetime import datetime
from typing import Union, List

from tautulli.api.json_api import RawAPI
from tautulli.internal.decorators import raw_api_bool, make_object, make_property_object
from tautulli.models import *
from tautulli.tools.api_helper import APIShortcuts


# noinspection PyTypeChecker,PyUnusedLocal
class ObjectAPI:
    def __init__(self, base_url: str, api_key: str, verbose: bool = False, verify: bool = True,
                 ssl_verify: bool = True):
        self._raw_api = RawAPI(base_url=base_url, api_key=api_key, verbose=verbose, verify=verify,
                               ssl_verify=ssl_verify)

    def __init_subclass__(cls, **kwargs):
        super().__init_subclass__(**kwargs)

    def __str__(self):
        return f"ObjectAPI(url={self._raw_api._redacted_url})"

    @property
    def shortcuts(self) -> APIShortcuts:
        """
        Shortcuts for common API actions

        :returns: Access to API shortcuts
        :rtype: APIShortcuts
        """
        return self._raw_api.shortcuts

    @raw_api_bool
    def add_newsletter_config(self, agent_id: int) -> bool:
        """
        Add a new newsletter notification agent

        :param agent_id: Newsletter type to add
        :type agent_id: int
        :returns: `True` if successful, `False` if unsuccessful
        :rtype: bool
        """
        return False

    @raw_api_bool
    def add_notifier_config(self, agent_id: int) -> bool:
        """
        Add a new notifier notification agent

        :param agent_id: Notification agent to add
        :type: int
        :returns: `True` if successful, `False` if unsuccessful
        :rtype: bool
        """
        return False

    @property
    def arnold(self) -> str:
        """
        Get to the chopper!

        :returns: Random Arnold Schwarzenegger quote
        :rtype: str
        """
        return self._raw_api.arnold

    @raw_api_bool
    def backup_config(self) -> bool:
        """
        Backup the config.ini file

        :returns: `True` if successful, `False` if unsuccessful
        :rtype: bool
        """
        return False

    @raw_api_bool
    def backup_database(self) -> bool:
        """
        Backup the plexpy.db database

        :returns: `True` if successful, `False` if unsuccessful
        :rtype: bool
        """
        return False

    @raw_api_bool
    def delete_all_library_history(self, server_id: str, section_id: str, row_ids: List[int] = None) -> bool:
        """
        Delete all Tautulli history for a specific library

        :param server_id: Plex server identifier of the library section
        :type server_id: str
        :param section_id: ID of the Plex library section
        :type section_id: str
        :param row_ids: List of row IDS to delete
        :type row_ids: list[int], optional
        :returns: `True` if successful, `False` if unsuccessful
        :rtype: bool
        """
        return False

    @raw_api_bool
    def delete_all_user_history(self, user_id: str, row_ids: List[int] = None) -> bool:
        """
        Delete all Tautulli history for a specific user

        :param user_id: ID of the Plex user
        :type user_id: str
        :param row_ids: List of row IDs to delete
        :type row_ids: list[int], optional
        :returns: `True` if successful, `False` if unsuccessful
        :rtype: bool
        """
        return False

    @raw_api_bool
    def delete_cache(self) -> bool:
        """
        Delete the cache directory

        :returns: `True` if successful, `False` if unsuccessful
        :rtype: bool
        """
        return False

    @raw_api_bool
    def delete_export(self, export_id: int, delete_all: bool = False) -> bool:
        """
        Delete exports from Tautulli

        :param export_id: Row ID of the exported file to delete
        :type export_id: int
        :param delete_all: Whether to delete all exported files (default: False)
        :type delete_all: bool
        :returns: `True` if successful, `False` if unsuccessful
        :rtype: bool
        """
        return False

    @raw_api_bool
    def delete_history(self, row_ids: List[int]) -> bool:
        """
        Delete history rows from Tautulli

        :param row_ids: List of row IDs to delete
        :type row_ids: list[int]
        :returns: `True` if successful, `False` if unsuccessful
        :rtype: bool
        """
        return False

    @raw_api_bool
    def delete_hosted_images(self, rating_key: int = None, service: str = None, delete_all: bool = False) -> bool:
        """
        Delete images uploaded to image hosting services

        :param rating_key: Rating key of image
        :type rating_key: int, optional
        :param service: Service to delete image from (i.e. 'imgur', 'cloudinary')
        :type service: str, optional
        :param delete_all: Whether to delete all images from the service (default: False)
        :type delete_all: bool, optional
        :returns: `True` if successful, `False` if unsuccessful
        :rtype: bool
        """
        return False

    @raw_api_bool
    def delete_image_cache(self) -> bool:
        """
        Delete image cache directory

        :returns: `True` if successful, `False` if unsuccessful
        :rtype: bool
        """
        return False

    @raw_api_bool
    def delete_library(self, server_id: str, section_id: str, row_ids: List[int] = None) -> bool:
        """
        Delete library section from Tautulli.

        Also erases library history.

        :param server_id: Plex server identifier of the library section
        :type server_id: str
        :param section_id: ID of the Plex library section
        :type section_id: str
        :param row_ids: List of row IDs to delete
        :type row_ids: list[int], optional
        :returns: `True` if successful, `False` if unsuccessful
        :rtype: bool
        """
        return False

    @raw_api_bool
    def delete_login_log(self) -> bool:
        """
        Delete the Tautulli login logs

        :returns: `True` if successful, `False` if unsuccessful
        :rtype: bool
        """
        return False

    @raw_api_bool
    def delete_lookup_info(self, rating_key: int = None, service: str = None, delete_all: bool = False) -> bool:
        """
        Delete the 3rd party API lookup info

        :param rating_key: rating key of image to delete
        :type rating_key: int, optional
        :param service: service to delete from (i.e. 'themoviedb', 'tvmaze', 'musicbrainz')
        :type service: str, optional
        :param delete_all: Whether to delete all images from the service (default: False)
        :type delete_all: bool, optional
        :returns: `True` if successful, `False` if unsuccessful
        :rtype: bool
        """
        return False

    @raw_api_bool
    def delete_media_info_cache(self, section_id: str) -> bool:
        """
        Delete media info table cache for a specific library

        :param section_id: ID of the Plex library section
        :type section_id: str
        :returns: `True` if successful, `False` if unsuccessful
        :rtype: bool
        """
        return False

    @raw_api_bool
    def delete_mobile_device(self, mobile_device_id: int = None, device_id: str = None) -> bool:
        """
        Remove a mobile device from the database

        :param mobile_device_id: Mobile device database ID to delete
        :type mobile_device_id: int, optional
        :param device_id: Unique device identifier for the mobile device
        :type device_id: str, optional
        :returns: `True` if successful, `False` if unsuccessful
        :rtype: bool
        """
        return False

    @raw_api_bool
    def delete_newsletter(self, newsletter_id: int) -> bool:
        """
        Remove a newsletter from the database

        :param newsletter_id: ID of the newsletter to delete
        :type newsletter_id: int
        :returns: `True` if successful, `False` if unsuccessful
        :rtype: bool
        """
        return False

    @raw_api_bool
    def delete_newsletter_log(self) -> bool:
        """
        Delete the Tautulli newsletter logs

        :returns: `True` if successful, `False` if unsuccessful
        :rtype: bool
        """
        return False

    @raw_api_bool
    def delete_notification_log(self) -> bool:
        """
        Delete the Tautulli notification logs

        :returns: `True` if successful, `False` if unsuccessful
        :rtype: bool
        """
        return False

    @raw_api_bool
    def delete_notifier(self, notifier_id: int) -> bool:
        """
        Remove a notifier from the database

        :param notifier_id: ID of the notifier to delete
        :type notifier_id: int
        :returns: `True` if successful, `False` if unsuccessful
        :rtype: bool
        """
        return False

    @raw_api_bool
    def delete_recently_added(self) -> bool:
        """
        Flush all the recently added items in the database

        :returns: `True` if successful, `False` if unsuccessful
        :rtype: bool
        """
        return False

    @raw_api_bool
    def delete_synced_item(self, client_id: str, sync_id: str) -> bool:
        """
        Delete a synced item from a device

        :param client_id: Client ID of the device to delete from
        :type client_id: str
        :param sync_id: Sync ID of the synced item
        :type sync_id: str
        :returns: `True` if successful, `False` if unsuccessful
        :rtype: bool
        """
        return False

    @raw_api_bool
    def delete_temp_sessions(self) -> bool:
        """
        Flush all temporary sessions in the database

        :returns: `True` if successful, `False` if unsuccessful
        :rtype: bool
        """
        return False

    @raw_api_bool
    def delete_user(self, user_id: str, row_ids: List[int] = None) -> bool:
        """
        Delete a user from Tautulli.
        Also erases all history of the user.

        :param user_id: ID of the Plex user
        :type user_id: str
        :param row_ids: List of row IDs to delete
        :type row_ids: list[int], optional
        :returns: `True` if successful, `False` if unsuccessful
        :rtype: bool
        """
        return False

    @property
    def docs_md(self) -> str:
        """
        Get the Tautulli API docs formatted with markdown

        :returns: API docs str
        :rtype: str
        """
        return self._raw_api.docs_md

    @property
    @make_property_object
    def docs(self) -> Docs:
        """
        Get the Tautulli API docs

        :returns: Docs object
        """
        return 'Docs'

    def download_config(self) -> str:
        """
        Download the Tautulli configuration file

        :returns: Config file string
        :rtype: str
        """
        return self._raw_api.download_config()

    def download_database(self) -> bytes:
        """
        Download the Tautulli database file

        :returns: Database file bytearray
        :rtype: bytearray
        """
        return self._raw_api.download_database()

    def download_export(self, export_id: int) -> bytes:
        """
        Download an exported metadata file

        :returns: Metadata file byte array
        :rtype: bytes
        """
        return self._raw_api.download_export(export_id=export_id)

    def download_log(self) -> bytes:
        """
        Download the Tautulli log file

        :returns: Log file bytearray
        :rtype: bytearray
        """
        return self._raw_api.download_log()

    def download_plex_log(self) -> bytes:
        """
        Download the Plex log file

        :returns: Log file bytearray
        :rtype: bytearray
        """
        return self._raw_api.download_plex_log()

    @raw_api_bool
    def edit_library(self, section_id: str, custom_thumb: str = None, custom_art: str = None,
                     keep_history: bool = True) -> bool:
        """
        Update a library section on Tautulli

        :param section_id: ID of the Plex library section
        :type section_id: str
        :param custom_thumb: URL of the custom library thumbnail
        :type custom_thumb: str, optional
        :param custom_art: ULR of the custom library background art
        :type custom_art: str, optional
        :param keep_history: Whether to keep library history (default: True)
        :type keep_history: bool, optional
        :returns: `True` if successful, `False` if unsuccessful
        :rtype: bool
        """
        return False

    @raw_api_bool
    def edit_user(self, user_id: str, friendly_name: str = None, custom_thumb: str = None, keep_history: bool = True,
                  allow_guest: bool = False) -> bool:
        """
        Update a user on Tautulli

        :param user_id: ID of the Plex user
        :type user_id: str
        :param friendly_name: Friendly name of the user
        :type friendly_name: str, optional
        :param custom_thumb: ULR of the custom user thumbnail
        :type custom_thumb: str, optional
        :param keep_history: Whether to keep user history (default: True)
        :type keep_history: bool, optional
        :param allow_guest: Whether to allow user as a guest (default: False)
        :type allow_guest: bool, optional
        :returns: `True` if successful, `False` if unsuccessful
        :rtype: bool
        """
        return False

    @raw_api_bool
    def export_metadata(self, section_id: int = None, user_id: int = None, rating_key: int = None,
                        file_format: str = 'csv', metadata_level: int = 1, media_info_level: int = 1,
                        thumb_level: int = 0, art_level: int = 0, custom_fields: List[str] = None,
                        export_type: str = None, individual_files: bool = False) -> bool:
        """
        Export library or media metadata to a file

        :param section_id: Section ID of the library items to export
        :type section_id: int, optional
        :param user_id: User ID of the playlist items to export
        :type user_id: int, optional
        :param rating_key: Rating key of the media items to export
        :type rating_key: int, optional
        :param file_format: File format for export (i.e. 'csv', 'json', 'xml', 'm3u') (default: 'csv')
        :type file_format: str, optional
        :param metadata_level: Level of metadata to export (default: 1)
        :type metadata_level: int, optional
        :param media_info_level: Level of media info to export (default: 1)
        :type media_info_level: int, optional
        :param thumb_level: Level of poster/cover images to export (default: 0)
        :type thumb_level: int, optional
        :param art_level: Level of background artwork images to export (default: 0)
        :type art_level: int, optional
        :param custom_fields: List of custom fields to export
        :type custom_fields: list[str], optional
        :param export_type: Type of export (i.e. 'collection' or 'playlist' for library/user export)
        :type export_type: str, optional
        :param individual_files: Export each item as an individual field for library/user export (default: False)
        :type individual_files: bool, optional
        :returns: `True` if successful, `False` if unsuccessful
        :rtype: bool
        """
        return False

    @make_object
    def activity(self, session_key: int = None, session_id: str = None) -> Activity:
        """
        Get the current activity on the Plex Media Server

        :param session_key: Session key for the session info to return
        :type session_key: int, optional
        :param session_id: Session ID of the session info to return
        :type session_id: str, optional
        :returns: Activity object
        """
        return 'Activity'

    def get_api_key(self, username: str = None, password: str = None) -> Union[str, None]:
        """
        Get the Tautulli API key.
        Username and password are required if auth is enabled.
        Makes and saves the API key if it does not exist.

        :param username: Tautulli username
        :type username: str, optional
        :param password: Tautulli password
        :type password: str, optional
        :returns: API key
        :rtype: str or None
        """
        return self._raw_api.get_api_key(username=username, password=password)

    @make_object
    def get_children_metadata(self, rating_key: str, media_type: str) -> ChildrenMetadata:
        """
        Get the metadata for the children of a media item.

        :param rating_key: The rating key of the media item
        :type rating_key: str
        :param media_type: The type of media item (movie, show, season, episode)
        :type media_type: str
        :returns: ChildrenMetadata object
        :rtype: ChildrenMetadata
        """
        return 'ChildrenMetadata'

    @make_object
    def get_collections_table(self, section_id: str) -> CollectionsTable:
        """
        Get the data on the Tautulli collections tables

        :param section_id: ID of the Plex library section
        :type section_id: str
        :returns: CollectionsTable object
        """
        return 'CollectionsTable'

    @property
    @make_property_object
    def date_formats(self) -> DateFormats:
        """
        Get the data and time formats used by Tautulli

        :returns: DateFormats object
        """
        return "DateFormats"

    @make_object
    def get_export_fields(self, media_type: str, sub_media_type: str = None) -> ExportFields:
        """
        Get a list of available custom export fields

        :param media_type: Media type of the fields to return
        :type media_type: str, optional
        :param sub_media_type: Child media type for collections (i.e. 'movie', 'show', 'video', 'audio', 'photo')
        :type sub_media_type: str, optional
        :returns: ExportFields
        """
        return 'ExportFields'

    @make_object
    def get_exports_table(self, section_id: str = None, user_id: str = None, rating_key: str = None,
                          order_column: str = None, order_direction: str = None, start: int = 0, length: int = 25,
                          search: str = None) -> ExportsTable:
        """
        Get the data on the Tautulli export tables

        :param section_id: ID of the Plex library section
        :type section_id: str, optional
        :param user_id: ID of the Plex user
        :type user_id: str, optional
        :param rating_key: Rating key of the exported item
        :type rating_key: str, optional
        :param order_column: Column to order data by (i.e. 'added_at', 'sort_title', 'last_played')
        :type order_column: str, optional
        :param order_direction: Direction to order the rows ('desc' or 'asc')
        :type order_direction: str, optional
        :param start: Row number to start from (default: 0)
        :type start: int, optional
        :param length: Number of items to return (default: 25)
        :type length: int, optional
        :param search: String to search for
        :type search: str, optional
        :returns: ExportsTable object
        :rtype: ExportsTable
        """
        return 'ExportsTable'

    @make_object
    def get_geoip_lookup(self, ip_address: str) -> GeoIPLookup:
        """
        Get the Geolocation info for an IP address

        :param ip_address: IP address to look up
        :type ip_address: str
        :returns: GeoIPLookup
        """
        return 'GeoIPLookup'

    @make_object
    def get_history(self, grouping: bool = False, include_activity: bool = False, user: str = None, user_id: int = None,
                    rating_key: int = None, parent_rating_key: int = None, grandparent_rating_key: int = None,
                    start_date: datetime = None, section_id: int = None, media_type: str = None,
                    transcode_decision: str = None, guid: str = None, order_column: str = None,
                    order_direction: str = None, start: int = 0, length: int = 25, search: str = None) -> History:
        """
        Get the Tautulli history

        :param grouping: Whether to group results (default: False)
        :type grouping: bool, optional
        :param include_activity: Whether to include activity (default: False)
        :type include_activity: bool, optional
        :param user: Name of user
        :type user: str, optional
        :param user_id: ID of user
        :type user_id: int, optional
        :param rating_key: Rating key of item
        :type rating_key: int, optional
        :param parent_rating_key: Parent rating key of item
        :type parent_rating_key: int, optional
        :param grandparent_rating_key: Grandparent rating key of item
        :type grandparent_rating_key: int, optional
        :param start_date: Date to start results from
        :type start_date: datetime, optional
        :param section_id: ID of section
        :type section_id: int, optional
        :param media_type: Media type (i.e. 'movie', 'episode', 'track', 'live', 'collection', 'playlist')
        :type media_type: str, optional
        :param transcode_decision: Transcode decision (i.e. 'direct play', 'copy', 'transcode')
        :type transcode_decision: str, optional
        :param guid: Plex GUID for an item (e.g. "com.plexapp.agents.thetvdb://121361/6/1")
        :type guid: str, optional
        :param order_column: Column to order data by (i.e. 'date', 'platform', 'full_title')
        :type order_column: str, optional
        :param order_direction: Direction to order the rows ('desc' or 'asc')
        :type order_direction: str, optional
        :param start: Row number to start from (default: 0)
        :type start: int, optional
        :param length: Number of items to return (default: 25)
        :type length: int, optional
        :param search: String to search for
        :type search: str, optional
        :returns: History object
        """
        return 'History'

    @make_object
    def get_home_stats(self, grouping: bool = False, time_range: int = 30, stats_type: str = 'plays', start: int = 0,
                       count: int = 5, stat_id: str = None, user_id: int = None, section_id: int = None,
                       before: datetime = None, after: datetime = None) -> List[HomeStat]:
        """
        Get the homepage watch statistics

        :param grouping: Whether to group results (default: False)
        :type grouping: bool, optional
        :param time_range: Time range to calculate statistics (i.e. 30)
        :type time_range: int, optional
        :param stats_type: Type of stats to get ('plays' or 'duration')
        :type stats_type: str, optional
        :param start: Row number to start from (default: 0)
        :type start: int, optional
        :param count: Number of items to return (default: 5)
        :type count: int, optional
        :param stat_id: Name of a single statistic to return (i.e. 'top_movies', 'popular_tv', 'most_concurrent')
        :type stat_id: str, optional
        :param user_id: The ID of the Plex user
        :type user_id: int, optional
        :param section_id: The ID of the Plex library section
        :type section_id: int, optional
        :param before: Results before and including the date
        :type before: datetime, optional
        :param after: Results after and including the date
        :type after: datetime, optional
        :returns: List of HomeStat object
        """
        return 'HomeStat'

    @make_object
    def get_item_user_stats(self, rating_key: str, grouping: bool = False, media_type: str = None) \
            -> List[ItemUserStat]:
        """
        Get the user statistics for the media item

        :param rating_key: Rating key of the media item
        :type rating_key: str
        :param grouping: Whether to group results (default: False)
        :type grouping: bool, optional
        :param media_type: Media type of the item (only required for a collection)
        :type media_type: str, optional
        :returns: List of ItemUserStat objects
        :rtype: List[ItemUserStat]
        """
        return 'ItemUserStat'

    @make_object
    def get_item_watch_time_stats(self, rating_key: str, grouping: bool = False, query_days: List[int] = None,
                                  media_type: str = None) \
            -> List[ItemWatchTimeStat]:
        """
        Get the watch time stats for the media item

        :param rating_key: Rating key of the media item
        :type rating_key: str
        :param grouping: Whether to group results (default: False)
        :type grouping: bool, optional
        :param query_days: List of days to get results for (i.e. [0, 1, 14, 30])
        :type query_days: list[int], optional
        :param media_type: Media type of the item (only required for a collection)
        :type media_type: str, optional
        :returns: List of ItemWatchTimeStat objects
        :rtype: List[ItemWatchTimeStat]
        """
        return 'ItemWatchTimeStat'

    @property
    @make_property_object
    def libraries(self) -> List[LibrariesEntry]:
        """
        Get a list of all libraries on your server

        :returns: List of LibrariesEntry objects

        """
        return 'LibrariesEntry'

    @make_object
    def get_libraries_table(self, grouping: bool = False, order_column: str = None, order_direction: str = None,
                            start: int = 0, length: int = 25, search: str = None) -> LibrariesTable:
        """
        Get the data on the Tautulli libraries table

        :param grouping: Whether to group results (default: False)
        :type grouping: bool, optional
        :param order_column: Column to order rows by (i.e. 'section_name', 'count', 'last_played')
        :type order_column: str, optional
        :param order_direction: Direction to order rows by ('desc' or 'asc')
        :type order_direction: str, optional
        :param start: Row number to start from (default: 0)
        :type start: int, optional
        :param length: Number of items to return (default: 25)
        :type length: int, optional
        :param search: String to search for
        :type search: str, optional
        :returns: LibrariesTable object

        """
        return 'LibrariesTable'

    @make_object
    def get_library(self, section_id: str) -> Library:
        """
        Get a library's details

        :param section_id: ID of the Plex library section
        :type section_id: str
        :returns: Library object

        """
        return 'Library'

    @make_object
    def get_library_media_info(self, section_id: str, rating_key: str, section_type: str = None,
                               order_column: str = None, order_direction: str = None, start: int = 0, length: int = 25,
                               search: str = None, refresh: bool = False) -> LibraryMediaInfo:
        """
        Get the data on the Tautulli media info tables.

        :param section_id: ID of the Plex library section
        :type section_id: str
        :param rating_key: Grandparent or parent rating key
        :type rating_key: str
        :param section_type: Type of section (i.e. 'movie', 'show', 'artist', 'photo')
        :type section_type: str
        :param order_column: Column to order rows by (i.e. 'added_at', 'sort_title', 'file_size')
        :type order_column: str, optional
        :param order_direction: Direction to order rows ('desc' or 'asc')
        :type order_direction: str, optional
        :param start: Row number to start from (default: 0)
        :type start: int, optional
        :param length: Number of items to return (default: 25)
        :type length: int, optional
        :param search: String to search for
        :type search: str, optional
        :param refresh: Whether to refresh the media info table (default: False)
        :type refresh: bool, optional
        :returns: LibraryMediaInfo object

        """
        return 'LibraryMediaInfo'

    @property
    @make_property_object
    def library_names(self) -> List[LibraryName]:
        """
        Get list of library names and IDs on the Plex Media Server

        :returns: List of LibraryName objects
        """
        return 'LibraryName'

    @make_object
    def get_library_user_stats(self, section_id: str, grouping: bool = False) -> LibraryUserStats:
        """
        Get a library's user statistics

        :param section_id: ID of the Plex library section
        :type section_id: str
        :param grouping: Whether to group results (default: False)
        :type grouping: bool, optional
        :returns: LibraryUserStats objects

        """
        return 'LibraryUserStats'

    @make_object
    def get_library_watch_time_stats(self, section_id: str, grouping: bool = False,
                                     query_days: List[int] = None) -> LibraryWatchTimeStats:
        """
        Get a library's watch time statistics

        :param section_id: ID of the Plex library section
        :type section_id: str
        :param grouping: Whether to group results (default: False)
        :type grouping: bool, optional
        :param query_days: List of days to get results for (i.e. [0, 1, 14, 30])
        :type query_days: list[int], optional
        :returns: LibraryWatchTimeStats object

        """
        return 'LibraryWatchTimeStats'

    @make_object
    def get_logs(self, sort: str = None, search: str = None, order_direction: str = None, regex: str = None,
                 start: int = None, end: int = None) -> List[LogEntry]:
        """
        Get the Tautulli logs

        :param sort: What to sort the logs by (i.e. 'time', 'thread', 'msg', 'loglevel')
        :type sort: str, optional
        :param search: String to search for
        :type search: str, optional
        :param order_direction: Direction to order rows ('desc' or 'asc')
        :type order_direction: str, optional
        :param regex: Regex string to search for
        :type regex: str, optional
        :param start: Row number to start from
        :type start: int, optional
        :param end: Row number to end at
        :type end: int, optional
        :returns: List of LogEntry objects

        """
        return 'LogEntry'

    @make_object
    def get_metadata(self, rating_key: str = None, sync_id: str = None) -> Metadata:
        """
        Get the metadata for a media item

        :param rating_key: Rating key of the media item
        :type rating_key: str, optional
        :param sync_id: Sync ID of a synced item
        :type sync_id: str, optional
        :returns: Metadata object

        """
        return 'Metadata'

    @make_object
    def get_new_rating_keys(self, rating_key: str, media_type: str) -> NewRatingKeys:
        """
        Get a list of new rating keys for the Plex Media Server of all the item's parent/children

        :param rating_key: Rating key of item
        :type rating_key: str
        :param media_type: Type of media (i.e. 'movie', 'show', 'episode', 'album', 'track')
        :type media_type: str
        :returns: NewRatingKeys object

        """
        return 'NewRatingKeys'

    @make_object
    def get_newsletter_config(self, newsletter_id: int) -> NewsletterConfig:
        """
        Get the configuration for an existing newsletter agent

        :param newsletter_id: ID of the newsletter
        :type newsletter_id: int
        :returns: NewsletterConfig object

        """
        return 'NewsletterConfig'

    @make_object
    def get_newsletter_log(self, order_column: str = None, order_direction: str = None, start: int = 0,
                           length: int = 25, search: str = None) -> NewsletterLog:
        """
        Get the data on the Tautulli newsletter logs table

        :param order_column: Column to order rows by (i.e. 'timestamp', 'newsletter_id', 'start_date')
        :type order_column: str, optional
        :param order_direction: Direction to order rows ('desc' or 'asc')
        :type order_direction: str, optional
        :param start: Row number to start from (default: 0)
        :type start: int, optional
        :param length: Number of items to return (default: 25)
        :type length: int, optional
        :param search: String to search for
        :type search: str, optional
        :returns: NewsletterLog object

        """
        return 'NewsletterLog'

    @property
    @make_property_object
    def newsletters(self) -> List[Newsletter]:
        """
        Get a list of configured newsletters

        :returns: List of Newsletter objects
        """
        return 'Newsletter'

    @make_object
    def get_notification_log(self, order_column: str = None, order_direction: str = None, start: int = 0,
                             length: int = 25, search: str = None) -> NotificationLog:
        """
        Get the data on the Tautulli notification logs table

        :param order_column: Column to order rows by (i.e. 'timestamp', 'agent_name', 'notifier_id')
        :type order_column: str, optional
        :param order_direction: Direction to order rows ('desc' or 'asc')
        :type order_direction: str, optional
        :param start: Row number to start from (default: 0)
        :type start: int, optional
        :param length: Number of items to return (default: 25)
        :type length: int, optional
        :param search: String to search for
        :type search: str, optional
        :returns: NotificationLog object

        """
        return 'NotificationLog'

    @make_object
    def get_notifier_config(self, notifier_id: int) -> NotifierConfig:
        """
        Get the configuration for an existing notification agent

        :param notifier_id: ID of the notifier
        :type notifier_id: int
        :returns: NotifierConfig object

        """
        return 'NotifierConfig'

    @property
    @make_property_object
    def notifier_parameters(self) -> List[NotifierParameter]:
        """
        Get a list of available notification parameters

        :returns: List of NotifierParameter objects
        """
        return 'NotifierParameter'

    @make_object
    def get_notifiers(self, notify_action: str = None) -> List[Notifier]:
        """
        Get a list of configured notifiers

        :param notify_action: The notification action to filter out
        :type notify_action: str, optional
        :returns: List of Notifier objects

        """
        return 'Notifier'

    @make_object
    def get_old_rating_keys(self, rating_key: str, media_type: str) -> OldRatingKeys:
        """
        Get a list of old rating keys from the Tautulli database for all the item's parent/children

        :param rating_key: Rating key of item
        :type rating_key: str
        :param media_type: Type of media (i.e. 'movie', 'show', 'episode', 'album', 'track')
        :type media_type: str
        :returns: OldRatingKeys object
        """
        return 'OldRatingKeys'

    @make_object
    def get_playlists_table(self, section_id: str = None, user_id: str = None) -> PlaylistsTable:
        """
        Get the data on the Tautulli playlists tables

        :param section_id: Section ID of the Plex library
        :type section_id: str, optional
        :param user_id: User ID of the Plex user
        :type user_id: str, optional
        :returns: PlaylistsTable object

        """
        return 'PlaylistsTable'

    @make_object
    def get_plays_by_date(self, time_range: int = None, y_axis: str = None, user_ids: List[str] = None,
                          grouping: bool = False) -> PlaysOrStreamTypesBy:
        """
        Get graph data by date

        :param time_range: Number of days of data to return
        :type time_range: int, optional
        :param y_axis: Stat type ('plays' or 'duration')
        :type y_axis: str, optional
        :param user_ids: List of user IDs to filter data
        :type user_ids: List[str], optional
        :param grouping: Whether to group the results (default: False)
        :type grouping: bool, optional
        :returns: PlaysOrStreamTypesBy object
        :rtype: PlaysOrStreamTypesBy
        """
        return 'PlaysOrStreamTypesBy'

    @make_object
    def get_plays_by_day_of_week(self, time_range: int = None, y_axis: str = None, user_ids: List[str] = None,
                                 grouping: bool = False) -> PlaysOrStreamTypesBy:
        """
        Get graph data by day of the week

        :param time_range: Number of days of data to return
        :type time_range: int, optional
        :param y_axis: Stat type ('plays' or 'duration')
        :type y_axis: str, optional
        :param user_ids: List of user IDs to filter data
        :type user_ids: List[str], optional
        :param grouping: Whether to group the results (default: False)
        :type grouping: bool, optional
        :returns: PlaysOrStreamTypesBy object
        :rtype: PlaysOrStreamTypesBy
        """
        return 'PlaysOrStreamTypesBy'

    @make_object
    def get_plays_by_hour_of_day(self, time_range: int = None, y_axis: str = None, user_ids: List[str] = None,
                                 grouping: bool = False) -> PlaysOrStreamTypesBy:
        """
        Get graph data by hour of the day

        :param time_range: Number of days of data to return
        :type time_range: int, optional
        :param y_axis: Stat type ('plays' or 'duration')
        :type y_axis: str, optional
        :param user_ids: List of user IDs to filter data
        :type user_ids: List[str], optional
        :param grouping: Whether to group the results (default: False)
        :type grouping: bool, optional
        :returns: PlaysOrStreamTypesBy object
        :rtype: PlaysOrStreamTypesBy
        """
        return 'PlaysOrStreamTypesBy'

    @make_object
    def get_plays_by_source_resolution(self, time_range: int = None, y_axis: str = None, user_ids: List[str] = None,
                                       grouping: bool = False) -> PlaysOrStreamTypesBy:
        """
        Get graph data by source resolution

        :param time_range: Number of days of data to return
        :type time_range: int, optional
        :param y_axis: Stat type ('plays' or 'duration')
        :type y_axis: str, optional
        :param user_ids: List of user IDs to filter data
        :type user_ids: List[str], optional
        :param grouping: Whether to group the results (default: False)
        :type grouping: bool, optional
        :returns: PlaysOrStreamTypesBy object
        :rtype: PlaysOrStreamTypesBy
        """
        return 'PlaysOrStreamTypesBy'

    @make_object
    def get_plays_by_stream_resolution(self, time_range: int = None, y_axis: str = None, user_ids: List[str] = None,
                                       grouping: bool = False) -> PlaysOrStreamTypesBy:
        """
        Get graph data by stream resolution

        :param time_range: Number of days of data to return
        :type time_range: int, optional
        :param y_axis: Stat type ('plays' or 'duration')
        :type y_axis: str, optional
        :param user_ids: List of user IDs to filter data
        :type user_ids: List[str], optional
        :param grouping: Whether to group the results (default: False)
        :type grouping: bool, optional
        :returns: PlaysOrStreamTypesBy object
        :rtype: PlaysOrStreamTypesBy
        """
        return 'PlaysOrStreamTypesBy'

    @make_object
    def get_plays_by_stream_type(self, time_range: int = None, y_axis: str = None, user_ids: List[str] = None,
                                 grouping: bool = False) -> PlaysOrStreamTypesBy:
        """
        Get graph data by stream type

        :param time_range: Number of days of data to return
        :type time_range: int, optional
        :param y_axis: Stat type ('plays' or 'duration')
        :type y_axis: str, optional
        :param user_ids: List of user IDs to filter data
        :type user_ids: List[str], optional
        :param grouping: Whether to group the results (default: False)
        :type grouping: bool, optional
        :returns: PlaysOrStreamTypesBy object
        :rtype: PlaysOrStreamTypesBy
        """
        return 'PlaysOrStreamTypesBy'

    @make_object
    def get_plays_by_top_10_platforms(self, time_range: int = None, y_axis: str = None, user_ids: List[str] = None,
                                      grouping: bool = False) -> PlaysOrStreamTypesBy:
        """
        Get graph data by top 10 platforms

        :param time_range: Number of days of data to return
        :type time_range: int, optional
        :param y_axis: Stat type ('plays' or 'duration')
        :type y_axis: str, optional
        :param user_ids: List of user IDs to filter data
        :type user_ids: List[str], optional
        :param grouping: Whether to group the results (default: False)
        :type grouping: bool, optional
        :returns: PlaysOrStreamTypesBy object
        :rtype: PlaysOrStreamTypesBy
        """
        return 'PlaysOrStreamTypesBy'

    @make_object
    def get_plays_by_top_10_users(self, time_range: int = None, y_axis: str = None, user_ids: List[str] = None,
                                  grouping: bool = False) -> PlaysOrStreamTypesBy:
        """
        Get graph data by top 10 users

        :param time_range: Number of days of data to return
        :type time_range: int, optional
        :param y_axis: Stat type ('plays' or 'duration')
        :type y_axis: str, optional
        :param user_ids: List of user IDs to filter data
        :type user_ids: List[str], optional
        :param grouping: Whether to group the results (default: False)
        :type grouping: bool, optional
        :returns: PlaysOrStreamTypesBy object
        :rtype: PlaysOrStreamTypesBy
        """
        return 'PlaysOrStreamTypesBy'

    @make_object
    def get_plays_per_month(self, time_range: int = None, y_axis: str = None, user_ids: List[str] = None,
                            grouping: bool = False) -> PlaysOrStreamTypesBy:
        """
        Get graph data by month

        :param time_range: Number of days of data to return
        :type time_range: int, optional
        :param y_axis: Stat type ('plays' or 'duration')
        :type y_axis: str, optional
        :param user_ids: List of user IDs to filter data
        :type user_ids: List[str], optional
        :param grouping: Whether to group the results (default: False)
        :type grouping: bool, optional
        :returns: PlaysOrStreamTypesBy object
        :rtype: PlaysOrStreamTypesBy
        """
        return 'PlaysOrStreamTypesBy'

    @make_object
    def get_plex_log(self, window: int = None, log_type: str = None) -> PlexLog:
        """
        Get the Plex Media Server logs

        :param window: Number of tail lines to return
        :type window: int, optional
        :param log_type: Log type ('server' or 'scanner')
        :type log_type: str, optional
        :returns: PlexLog object

        """
        return 'PlexLog'

    @property
    @make_property_object
    def pms_update(self) -> PMSUpdate:
        """
        Check for updates to the Plex Media Server

        :returns: PMSUpdate object
        """
        return 'PMSUpdate'

    @make_object
    def get_recently_added(self, count: int, start: int = 0, media_type: str = None,
                           section_id: str = None) -> RecentlyAdded:
        """
        Get all items that were recently added to Plex

        :param count: Number of item to return
        :type count: int
        :param start: Item number to start from
        :type start: int, optional
        :param media_type: Media type (i.e. 'movie', 'show', 'artist')
        :type media_type: str, optional
        :param section_id: ID of the Plex library section
        :type section_id: str, optional
        :returns: RecentlyAdded object
        """
        return 'RecentlyAdded'

    @property
    def server_friendly_name(self) -> str:
        """
        Get the name of the Plex Media Server

        :returns: Name of the Plex Media Server
        :rtype: str
        """
        return self._raw_api.server_friendly_name

    @make_object
    def get_server_id(self, hostname: str, port: int, ssl: bool = False, remote: bool = False) -> ServerID:
        """
        Get the Plex Media Server identifier

        :param hostname: IP address of the Plex Media Server
        :type hostname: str
        :param port: Port of the Plex Media Server
        :type port: int
        :param ssl: Whether to use SSL (default: False)
        :type ssl: bool, optional
        :param remote: Whether the Plex Media Server is remote (default: False)
        :type remote: bool, optional
        :returns: ServerID object

        """
        return 'ServerID'

    @property
    @make_property_object
    def server_identity(self) -> ServerIdentity:
        """
        Get info about the local server

        :returns: ServerIdentity object
        """
        return 'ServerIdentity'

    @property
    @make_property_object
    def server_info(self) -> ServerInfo:
        """
        Get the Plex Media Server information

        :returns: ServerInfo object
        """
        return 'ServerInfo'

    @property
    @make_property_object
    def server_list(self) -> List[ServerListEntry]:
        """
        Get all your servers that are published to Plex.tv

        :returns: List of ServerListEntry objects

        """
        return 'ServerListEntry'

    def get_server_pref(self, pref: str) -> str:
        """
        Get a specified Plex Media Server preference

        :param pref: Name of preference
        :type pref: str
        :returns: Value of preference
        :rtype: str
        """
        return self._raw_api.get_server_pref(pref=pref)

    @property
    @make_property_object
    def servers_info(self) -> List[ServersInfoEntry]:
        """
        Get info about the Plex Media Server

        :returns: List of ServersInfoEntry objects

        """
        return 'ServersInfoEntry'

    @make_object
    def get_settings(self, key: str = None) -> Settings:
        """
        Get all settings from the config file

        :param key: Name of a config section to return
        :type key: str, optional
        :returns: Settings object

        """
        return 'Settings'

    @make_object
    def get_stream_data(self, row_id: int = None, session_key: int = None) -> StreamData:
        """
        Get the details of a stream from history or current stream

        :param row_id: Row ID number for a history item
        :type row_id: int, optional
        :param session_key: Session key for the current stream
        :type session_key: int, optional
        :returns: StreamData object

        """
        return 'StreamData'

    @make_object
    def get_stream_type_by_top_10_platforms(self, time_range: int = None, y_axis: str = None,
                                            user_ids: List[str] = None,
                                            grouping: bool = False) -> PlaysOrStreamTypesBy:
        """
        Get graph data by stream type by the top 10 platforms

        :param time_range: Number of days of data to return
        :type time_range: int, optional
        :param y_axis: Stat type ('plays' or 'duration')
        :type y_axis: str, optional
        :param user_ids: List of user IDs to filter data
        :type user_ids: List[str], optional
        :param grouping: Whether to group the results (default: False)
        :type grouping: bool, optional
        :returns: PlaysOrStreamTypesBy object
        :rtype: PlaysOrStreamTypesBy
        """
        return 'PlaysOrStreamTypesBy'

    @make_object
    def get_stream_type_by_top_10_users(self, time_range: int = None, y_axis: str = None, user_ids: List[str] = None,
                                        grouping: bool = False) -> PlaysOrStreamTypesBy:
        """
        Get graph data by stream type by the top 10 users

        :param time_range: Number of days of data to return
        :type time_range: int, optional
        :param y_axis: Stat type ('plays' or 'duration')
        :type y_axis: str, optional
        :param user_ids: List of user IDs to filter data
        :type user_ids: List[str], optional
        :param grouping: Whether to group the results (default: False)
        :type grouping: bool, optional
        :returns: PlaysOrStreamTypesBy object
        :rtype: PlaysOrStreamTypesBy
        """
        return 'PlaysOrStreamTypesBy'

    @make_object
    def get_synced_items(self, machine_id: str, user_id: str = None) -> SyncedItems:
        """
        Get a list of synced items on the Plex Media Server

        :param machine_id: Plex Media Server identifier
        :type machine_id: str
        :param user_id: ID of the Plex user
        :type user_id: str, optional
        :returns: SyncedItems object

        """
        return 'SyncedItems'

    @property
    @make_property_object
    def tautulli_info(self) -> TautulliInfo:
        """
        Get the Tautulli server information

        :returns: Dict of data
        :rtype: dict
        """
        return 'TautulliInfo'

    @make_object
    def get_user(self, user_id: str) -> User:
        """
        Get a user's details

        :param user_id: ID of the Plex user
        :type user_id: str
        :returns: User object

        """
        return 'User'

    @make_object
    def get_user_ips(self, user_id: str, order_column: str = None, order_direction: str = None, start: int = 0,
                     length: int = 25, search: str = None) -> UserIPs:
        """
        Get the data on Tautulli's users IP table

        :param user_id: ID of the Plex user
        :type user_id: str
        :param order_column: Column to order rows by (i.e. 'last_seen', 'ip_address', 'player')
        :type order_column: str, optional
        :param order_direction: Direction to order rows ('desc' or 'asc')
        :type order_direction: str, optional
        :param start: Row number to start from (default: 0)
        :type start: int, optional
        :param length: Number of items to return (default: 25)
        :type length: int, optional
        :param search: String to search for (e.g. "xxx.xxx.xxx.xxx")
        :type search: str, optional
        :returns: UserIPs object

        """
        return 'UserIPs'

    @make_object
    def get_user_logins(self, user_id: str, order_column: str = None, order_direction: str = None, start: int = 0,
                        length: int = 25, search: str = None) -> UserLogins:
        """
        Get the data on Tautulli's user login table

        :param user_id: ID of the Plex user
        :type user_id: str
        :param order_column: Column to order rows by (i.e. 'date', 'time', 'ip_address')
        :type order_column: str, optional
        :param order_direction: Direction to order rows ('desc' or 'asc')
        :type order_direction: str, optional
        :param start: Row number to start from (default: 0)
        :type start: int, optional
        :param length: Number of items to return (default: 25)
        :type length: int, optional
        :param search: String to search for (e.g. "xxx.xxx.xxx.xxx")
        :type search: str, optional
        :returns: UserLogins object

        """
        return 'UserLogins'

    @property
    @make_property_object
    def user_names(self) -> List[UserName]:
        """
        Get a list of all usernames and user ids

        :returns: List of UserName objects

        """
        return 'UserName'

    @make_object
    def get_user_player_stats(self, user_id: str, grouping: bool = False) -> UserPlayerStats:
        """
        Get a user's player statistics

        :param user_id: ID of the Plex user
        :type user_id: str
        :param grouping: Whether to group results (default: False)
        :type grouping: bool, optional
        :returns: UserPlayerStats object

        """
        return 'UserPlayerStats'

    @make_object
    def get_user_watch_time_stats(self, user_id: str, grouping: bool = False,
                                  query_days: List[int] = None) -> UserWatchTimeStats:
        """
        Get a user's watch time statistics

        :param user_id: ID of the Plex user
        :type user_id: str
        :param grouping: Whether to group results (default: False)
        :type grouping: bool, optional
        :param query_days: List of days to get results for (e.g. [0, 1, 14, 30])
        :type query_days: list[int], optional
        :returns: UserWatchTimeStats object

        """
        return 'UserWatchTimeStats'

    @property
    @make_property_object
    def users(self) -> List[User]:
        """
        Get a list of all users that have access to your server

        :returns: List of User objects

        """
        return 'User'

    @make_object
    def get_users_table(self, grouping: bool = False, order_column: str = None, order_direction: str = None,
                        start: int = 0, length: int = 25, search: str = None) -> UsersTable:
        """
        Get the data on Tautulli's users table

        :param grouping: Whether to group results (default: False)
        :type grouping: bool, optional
        :param order_column: Column to order rows by ('friendly_name', 'ip_address', 'player')
        :type order_column: str, optional
        :param order_direction: Direction to order rows ('desc' or 'asc')
        :type order_direction: str, optional
        :param start: Row number to start from (default: 0)
        :type start: int, optional
        :param length: Number of items to return (default: 25)
        :type length: int, optional
        :param search: String to search for
        :type search: str, optional
        :returns: UsersTable object

        """
        return 'UsersTable'

    @make_object
    def get_whois_lookup(self, ip_address: str) -> WHOISLookup:
        """
        Get the connection info for an IP address

        :param ip_address: IP address
        :type ip_address: str
        :returns: WHOISLookup object

        """
        return 'WHOISLookup'

    @raw_api_bool
    def import_config(self, config_file_path: str, backup: bool = True) -> bool:
        """
        Import a Tautulli config file

        :param config_file_path: Full path to the config file to import
        :type config_file_path: str
        :param backup: Whether to back up the current config before importing (default: True)
        :type backup: bool, optional
        :returns: `True` if successful, `False` if unsuccessful
        :rtype: bool
        """
        return False

    @raw_api_bool
    def import_database(self, app: str, database_file_path: str, method: str = None, table_name: str = None,
                        backup: bool = True, import_ignore_interval: int = None) -> bool:
        """
        Import a Tautulli, PlexWatch or Plexivity database into Tautulli

        :param app: Type of app the database is from ('tautulli', 'plexwatch' or 'plexivity')
        :type app: str
        :param database_file_path: Full path to the database file to import
        :type database_file_path: str
        :param method: Only if app is 'tautulli', method to import database ('merge' or 'overwrite')
        :type method: str, optional
        :param table_name: Only if app is 'plexwatch' or 'plexivity', table name to import ('processed' or 'grouped')
        :type table_name: str, optional
        :param backup: Whether to back up the current database before importing (default: True)
        :type backup: bool, optional
        :param import_ignore_interval: Only if app is 'plexwatch' or 'plexivity', the minimum number of seconds for a stream to import
        :type import_ignore_interval: int, optional
        :returns: `True` if successful, `False` if unsuccessful
        :rtype: bool
        """
        return False

    @raw_api_bool
    def logout_user_session(self, row_ids: List[int]) -> bool:
        """
        Logout Tautulli user sessions

        :param row_ids: List of row IDS to sign out
        :type row_ids: list[int], optional
        :returns: `True` if successful, `False` if unsuccessful
        :rtype: bool
        """
        return False

    @make_object
    def notify(self, notifier_id: int, subject: str, body: str, headers: str = None,
               script_args: str = None) -> Notification:
        """
        Send a notification using Tautulli

        :param notifier_id: ID of the notification agent
        :type notifier_id: int
        :param subject: Subject of the message
        :type subject: str
        :param body: Body of the message
        :type body: str
        :param headers: JSON headers for webhook notifications
        :type headers: str, optional
        :param script_args: Arguments for script notifications
        :type script_args: str, optional
        :returns: Notification object

        """
        return 'Notification'

    @make_object
    def notify_newsletter(self, newsletter_id: int, subject: str = None, body: str = None,
                          message: str = None) -> NewsletterNotification:
        """
        Send a newsletter using Tautulli

        :param newsletter_id: ID of the newsletter agent
        :type newsletter_id: int
        :param subject: Subject of the newsletter
        :type subject: str, optional
        :param body: Body of the newsletter
        :type body: str, optional
        :param message: Message of the newsletter
        :type message: str, optional
        :returns: NewsletterNotification object

        """
        return 'NewsletterNotification'

    @raw_api_bool
    def notify_recently_added(self, rating_key: int, notifier_id: int = None) -> bool:
        """
        Send a recently added notification using Tautulli

        :param rating_key: Rating key for the media item
        :type rating_key: int
        :param notifier_id: ID of the notification agent. Notification will be sent to all enabled notification agents if notifier_id is not provided.
        :type notifier_id: int, optional
        :returns: `True` if successful, `False` if unsuccessful
        :rtype: bool
        """
        return False

    def pms_image_proxy(self, img: str = None, rating_key: str = None, width: int = None, height: int = None,
                        opacity: int = None, background_hex: str = None, blur: int = None, img_format: str = 'png',
                        fallback: str = None, refresh: bool = False) -> bool:
        """
        Gets an image from the Plex Media Server and saves it to the image cache directory

        :param img: Path of image (e.g. "/library/metadata/153037/thumb/1462175060")
        :type img: str, optional
        :param rating_key: Rating key of image
        :type rating_key: str, optional
        :param width: Width of image
        :type width: int, optional
        :param height: Height of image
        :type height: int, optional
        :param opacity: Opacity of image
        :type opacity: int, optional
        :param background_hex: Background hex color
        :type background_hex: str, optional
        :param blur: Blur level of image
        :type blur: int, optional
        :param img_format: Format of image (i.e. 'png')
        :type img_format: str, optional
        :param fallback: Fallback of image (i.e. 'poster', 'cover', 'art', 'poster-live', 'art-live', 'art-live-full')
        :type fallback: str, optional
        :param refresh: Whether or refresh the image cache (default: False)
        :type refresh: bool, optional
        :returns: Bytes of image
        :rtype: bytearray
        """
        return self._raw_api.pms_image_proxy(img=img, rating_key=rating_key, width=width, height=height,
                                             opacity=opacity,
                                             background_hex=background_hex, blur=blur, img_format=img_format,
                                             fallback=fallback, refresh=refresh)

    @raw_api_bool
    def refresh_libraries_list(self) -> bool:
        """
        Refresh the Tautulli libraries list

        :returns: `True` if successful, `False` if unsuccessful
        :rtype: bool
        """
        return False

    @raw_api_bool
    def refresh_users_list(self) -> bool:
        """
        Refresh the Tautulli users list

        :returns: `True` if successful, `False` if unsuccessful
        :rtype: bool
        """
        return False

    @make_object
    def register_device(self, device_id: str, device_name: str, friendly_name: str = None, onesignal_id: str = None,
                        min_version: str = None) -> RegisteredDevice:
        """
        Register the Tautulli Android App for notifications

        :param device_id: Unique device identifier for the mobile device
        :type device_id: str
        :param device_name: Device name of the mobil device
        :type device_name: str
        :param friendly_name: Friendly name to identity the mobile device
        :type friendly_name: str, optional
        :param onesignal_id: The OneSignal ID of the mobile device
        :type onesignal_id: str, optional
        :param min_version: The minimum Tautulli version supported by the mobile device (e.g. "v2.5.6")
        :type min_version: str, optional
        :returns: RegisteredDevice object

        """
        return 'RegisteredDevice'

    @raw_api_bool
    def regroup_history(self) -> bool:
        """
        Regroup play history in the database

        :returns: `True` if successful, `False` if unsuccessful
        :rtype: bool
        """
        return False

    @raw_api_bool
    def restart(self) -> bool:
        """
        Restart Tautulli

        :returns: `True` if successful, `False` if unsuccessful
        :rtype: bool
        """
        return False

    @make_object
    def search(self, query: str, limit: int = None) -> SearchResults:
        """
        Get search results from the Plex Media Server

        :param query: String to search for
        :type query: str
        :param limit: Maximum number of items to return per media type
        :type limit: int, optional
        :returns: SearchResults object

        """
        return 'SearchResults'

    @property
    @make_property_object
    def server_status(self) -> ServerStatus:
        """
        Get the current status of Tautulli's connection to the Plex server

        :returns: ServerStatus object
        :rtype: ServerStatus
        """
        return 'ServerStatus'

    @raw_api_bool
    def set_mobile_device_config(self, mobile_device_id: int, friendly_name: str = None) -> bool:
        """
        Configure an existing notification agent

        :param mobile_device_id: ID of the mobile device config to update
        :type mobile_device_id: int
        :param friendly_name: Friendly name to identify the mobile device
        :type friendly_name: str, optional
        :returns: `True` if successful, `False` if unsuccessful
        :rtype: bool
        """
        return False

    @raw_api_bool
    def set_newsletter_config(self, newsletter_id: int, agent_id: int, **kwargs) -> bool:
        """
        Configure an existing newsletter agent

        :param newsletter_id: ID of the newsletter config to update
        :type newsletter_id: int
        :param agent_id: Type of the newsletter to update
        :type agent_id: int
        :returns: `True` if successful, `False` if unsuccessful
        :rtype: bool
        """
        return False

    @raw_api_bool
    def set_notifier_config(self, agent: str, notifier_id: int, agent_id: int, **kwargs) -> bool:
        """
        Configure an existing notification agent

        :param agent: Type of the notification agent to update
        :type agent: str
        :param notifier_id: ID of the notifier config to update
        :type notifier_id: int
        :param agent_id: Agent of the notifier to update
        :type agent_id: int
        :returns: `True` if successful, `False` if unsuccessful
        :rtype: bool
        """
        return False

    # TODO: Test to see what this returns, no schema on API docs
    @make_object
    def sql(self, query: str) -> None:
        """
        Query the Tautulli database with raw SQL.

        Automatically makes a backup of the database if the latest backup is older than 24 hours.

        `api_sql` must be manually enabled in the config file while Tautulli is shut down.

        :param query: SQL query
        :type query: str
        :returns: SQLResults object

        """
        warnings.warn("Deprecated", DeprecationWarning)
        return None
        # return 'SQLResults'

    @make_object
    def status(self, check: str = None) -> Status:
        """
        Get the current status of Tautulli

        :param check: What to check (i.e. 'database')
        :type check: str, optional
        :returns: Status object
        :rtype: Status
        """
        return 'Status'

    @raw_api_bool
    def terminate_session(self, session_key: int = None, session_id: str = None, message: str = None) -> bool:
        """
        Stop a streaming session

        :param session_key: Session key of the session to terminate
        :type session_key: int, optional
        :param session_id: Session ID of the session to terminate
        :type session_id: str, optional
        :param message: Custom message to send to the client
        :type message: str, optional
        :returns: `True` if successful, `False` if unsuccessful
        :rtype: bool
        """
        return False

    @raw_api_bool
    def undelete_library(self, section_id: str, section_name: str) -> bool:
        """
        Restore a deleted library section to Tautulli

        :param section_id: ID of the Plex library section
        :type section_id: str
        :param section_name: Name of the Plex library section
        :type section_name: str, optional
        :returns: `True` if successful, `False` if unsuccessful
        :rtype: bool
        """
        return False

    @raw_api_bool
    def undelete_user(self, user_id: str, username: str) -> bool:
        """
        Restore a deleted user to Tautulli

        :param user_id: ID of the Plex user
        :type user_id: str
        :param username: Username of the Plex user
        :type username: str
        :returns: `True` if successful, `False` if unsuccessful
        :rtype: bool
        """
        return False

    @raw_api_bool
    def update(self) -> bool:
        """
        Update Tautulli

        :returns: `True` if successful, `False` if unsuccessful
        :rtype: bool
        """
        return False

    @property
    @make_property_object
    def update_check(self) -> UpdateCheck:
        """
        Check for Tautulli updates

        :returns: UpdateCheck object

        """
        return 'UpdateCheck'

    @raw_api_bool
    def update_metadata_details(self, old_rating_key: str, new_rating_key: str, media_type: str) -> bool:
        """
        Update the metadata in the Tautulli database by matching rating keys.

        Also updates all parents or children of the media item if it is a show/season/episode or artist/album/track.

        :param old_rating_key: Old rating key for item
        :type old_rating_key: str
        :param new_rating_key: New rating key for item
        :type new_rating_key: str
        :param media_type: Type of media (i.e. 'movie', 'show', 'episode', 'album', 'track')
        :type media_type: str
        :returns: `True` if successful, `False` if unsuccessful
        :rtype: bool
        """
        return False



# generated by datamodel-codegen:
#   filename:  data.json
#   timestamp: 2022-08-26T22:18:10+00:00

from __future__ import annotations

from typing import Any, Optional

from pydantic import BaseModel


class ServerStatusModel(BaseModel):
    result: Optional[str] = None
    connected: Optional[bool] = None






# generated by datamodel-codegen:
#   filename:  data.json
#   timestamp: 2021-01-27T04:35:34+00:00

from __future__ import annotations

from typing import Any, List, Optional, Union

from pydantic import BaseModel


class RowModel(BaseModel):
    title: Optional[str] = None
    users_watched: Optional[Union[int, str]] = None
    rating_key: Optional[Union[int, str]] = None
    last_play: Optional[int] = None
    total_plays: Optional[int] = None
    grandparent_thumb: Optional[str] = None
    thumb: Optional[str] = None
    art: Optional[str] = None
    section_id: Optional[int] = None
    media_type: Optional[str] = None
    content_rating: Optional[str] = None
    labels: Optional[List] = None
    user: Optional[str] = None
    friendly_name: Optional[str] = None
    platform: Optional[str] = None
    live: Optional[int] = None
    guid: Optional[str] = None
    row_id: Optional[Union[int, str]] = None
    total_duration: Optional[int] = None
    count: Optional[int] = None
    started: Optional[str] = None
    stopped: Optional[str] = None
    user_id: Optional[int] = None
    user_thumb: Optional[str] = None
    platform_name: Optional[str] = None
    last_watch: Optional[int] = None
    player: Optional[str] = None


class HomeStatModel(BaseModel):
    stat_id: Optional[str] = None
    stat_title: Optional[str] = None
    rows: Optional[List[RowModel]] = None
    stat_type: Optional[str] = None






from logging import Logger

import objectrest

from tautulli.exceptions.base_exception import TautulliException


class HttpException(TautulliException):
    def __init__(self, message: str, status_code: int = None):
        super().__init__(message=message)
        self.status_code = status_code

    def __str__(self):
        return f"{self.message} (status code: {self.status_code})"

    def __repr__(self):
        return f"<{self.__class__.__name__}(message={self.message}, status_code={self.status_code})>"

    @classmethod
    def from_response(cls, response: objectrest.Response, logger: Logger) -> 'HttpException':
        tautulli_error = None

        try:
            data = response.json()
            tautulli_error = data.get('response', {}).get('message', None)
        except:
            logger.error(f"Could not parse JSON from API error response: {response.text}")

        if not tautulli_error:
            return HttpException(status_code=response.status_code, message="No error reason provided by Tautulli.")

        raise HttpException(status_code=response.status_code, message=tautulli_error)



class TautulliException(Exception):
    def __init__(self, message: str):
        Exception.__init__(self)
        self.message = message

    def __str__(self):
        return f"{self.message}"

    def __repr__(self):
        return f"<{self.__class__.__name__}(message={self.message})>"



# generated by datamodel-codegen:
#   filename:  data.json
#   timestamp: 2021-01-27T04:50:23+00:00

from __future__ import annotations

from typing import Any, Optional

from pydantic import BaseModel, Field


class Field0Model(BaseModel):
    rating_key: Optional[int] = None


class OldRatingKeysModel(BaseModel):
    field_0: Optional[Field0Model] = Field(..., alias='0')






# generated by datamodel-codegen:
#   filename:  data.json
#   timestamp: 2021-01-27T04:44:54+00:00

from __future__ import annotations

from typing import List, Optional, Union

from pydantic import BaseModel, Field


class ConfigModel(BaseModel):
    custom_cron: Optional[int] = None
    time_frame: Optional[int] = None
    time_frame_units: Optional[str] = None
    formatted: Optional[int] = None
    threaded: Optional[int] = None
    notifier_id: Optional[int] = None
    filename: Optional[str] = None
    save_only: Optional[int] = None
    incl_libraries: Optional[List[str]] = None


class EmailConfigModel(BaseModel):
    from_name: Optional[str] = None
    from_: Optional[str] = Field(..., alias='from')
    to: Optional[List[str]] = None
    cc: Optional[List] = None
    bcc: Optional[List] = None
    smtp_server: Optional[str] = None
    smtp_port: Optional[int] = None
    smtp_user: Optional[str] = None
    smtp_password: Optional[str] = None
    tls: Optional[int] = None
    html_support: Optional[int] = None
    notifier_id: Optional[int] = None


class MovieLibraryModel(BaseModel):
    value: Optional[int] = None
    text: Optional[str] = None


class TVShowLibraryModel(BaseModel):
    value: Optional[int] = None
    text: Optional[str] = None


class MusicLibraryModel(BaseModel):
    value: Optional[int] = None
    text: Optional[str] = None


class OtherVideoLibraryModel(BaseModel):
    value: Optional[int] = None
    text: Optional[str] = None


class SelectOptionsModel(BaseModel):
    Movie_Libraries: Optional[List[MovieLibraryModel]] = Field(..., alias='Movie Libraries')
    TV_Show_Libraries: Optional[List[TVShowLibraryModel]] = Field(..., alias='TV Show Libraries')
    Music_Libraries: Optional[List[MusicLibraryModel]] = Field(..., alias='Music Libraries')
    Other_Video_Libraries: Optional[List[OtherVideoLibraryModel]] = Field(
        ..., alias='Other Video Libraries'
    )


class ConfigOptionModel(BaseModel):
    label: Optional[str] = None
    value: Optional[List[str]] = None
    description: Optional[str] = None
    name: Optional[str] = None
    input_type: Optional[str] = None
    select_options: Optional[SelectOptionsModel] = None


class SelectOptionModel(BaseModel):
    value: Optional[str] = None
    text: Optional[str] = None


class EmailConfigOptionModel(BaseModel):
    label: Optional[str] = None
    value: Optional[Union[Union[int, str], List[str]]] = None
    name: Optional[str] = None
    description: Optional[str] = None
    input_type: Optional[str] = None
    select_options: Optional[List[SelectOptionModel]] = None


class NewsletterConfigModel(BaseModel):
    id: Optional[int] = None
    agent_id: Optional[int] = None
    agent_name: Optional[str] = None
    agent_label: Optional[str] = None
    friendly_name: Optional[str] = None
    cron: Optional[str] = None
    active: Optional[int] = None
    id_name: Optional[str] = None
    subject: Optional[str] = None
    body: Optional[str] = None
    config: Optional[ConfigModel] = None
    email_config: Optional[EmailConfigModel] = None
    config_options: Optional[List[ConfigOptionModel]] = None
    email_config_options: Optional[List[EmailConfigOptionModel]] = None






# generated by datamodel-codegen:
#   filename:  data.json
#   timestamp: 2022-02-20T07:32:48+00:00

from __future__ import annotations

from typing import Optional

from pydantic import BaseModel


class TautulliInfoModel(BaseModel):
    tautulli_install_type: Optional[str] = None
    tautulli_version: Optional[str] = None
    tautulli_branch: Optional[str] = None
    tautulli_commit: Optional[str] = None
    tautulli_platform: Optional[str] = None
    tautulli_platform_release: Optional[str] = None
    tautulli_platform_version: Optional[str] = None
    tautulli_platform_linux_distro: Optional[str] = None
    tautulli_platform_device_name: Optional[str] = None
    tautulli_python_version: Optional[str] = None



# generated by datamodel-codegen:
#   filename:  data.json
#   timestamp: 2022-08-26T22:25:38+00:00

from __future__ import annotations

from typing import Any, List, Optional

from pydantic import BaseModel


class ChildrenListItemModel(BaseModel):
    actors: Optional[List] = None
    added_at: Optional[str] = None
    art: Optional[str] = None
    audience_rating: Optional[str] = None
    audience_rating_image: Optional[str] = None
    banner: Optional[str] = None
    content_rating: Optional[str] = None
    directors: Optional[List] = None
    duration: Optional[str] = None
    full_title: Optional[str] = None
    collections: Optional[List] = None
    genres: Optional[List] = None
    grandparent_rating_key: Optional[str] = None
    grandparent_thumb: Optional[str] = None
    grandparent_title: Optional[str] = None
    guid: Optional[str] = None
    labels: Optional[List] = None
    last_viewed_at: Optional[str] = None
    library_name: Optional[str] = None
    media_index: Optional[str] = None
    media_type: Optional[str] = None
    original_title: Optional[str] = None
    originally_available_at: Optional[str] = None
    parent_media_index: Optional[str] = None
    parent_rating_key: Optional[str] = None
    parent_thumb: Optional[str] = None
    parent_title: Optional[str] = None
    rating: Optional[str] = None
    rating_image: Optional[str] = None
    rating_key: Optional[str] = None
    section_id: Optional[str] = None
    sort_title: Optional[str] = None
    studio: Optional[str] = None
    summary: Optional[str] = None
    tagline: Optional[str] = None
    thumb: Optional[str] = None
    title: Optional[str] = None
    updated_at: Optional[str] = None
    user_rating: Optional[str] = None
    writers: Optional[List] = None
    year: Optional[str] = None
    collections: Optional[List] = None


class ChildrenMetadataModel(BaseModel):
    children_count: Optional[int] = None
    children_type: Optional[str] = None
    title: Optional[str] = None
    children_list: Optional[List[ChildrenListItemModel]] = None






# generated by datamodel-codegen:
#   filename:  data.json
#   timestamp: 2021-01-27T05:05:45+00:00

from __future__ import annotations

from typing import Any, List, Optional, Union

from pydantic import BaseModel


class DatumModel(BaseModel):
    row_id: Optional[int] = None
    user_id: Optional[int] = None
    username: Optional[str] = None
    friendly_name: Optional[str] = None
    user_thumb: Optional[str] = None
    plays: Optional[int] = None
    duration: Optional[int] = None
    last_seen: Optional[int] = None
    last_played: Optional[str] = None
    history_row_id: Optional[int] = None
    ip_address: Optional[str] = None
    platform: Optional[str] = None
    player: Optional[str] = None
    rating_key: Optional[int] = None
    media_type: Optional[str] = None
    thumb: Optional[str] = None
    parent_title: Optional[str] = None
    year: Optional[Any] = None
    media_index: Optional[Union[int, str]] = None
    parent_media_index: Optional[Union[int, str]] = None
    live: Optional[int] = None
    originally_available_at: Optional[str] = None
    guid: Optional[str] = None
    transcode_decision: Optional[str] = None
    do_notify: Optional[int] = None
    keep_history: Optional[int] = None
    allow_guest: Optional[int] = None
    is_active: Optional[int] = None
    title: Optional[str] = None
    email: Optional[str] = None


class UsersTableModel(BaseModel):
    recordsFiltered: Optional[int] = None
    recordsTotal: Optional[int] = None
    data: Optional[List[DatumModel]] = None
    draw: Optional[int] = None






# generated by datamodel-codegen:
#   filename:  data.json
#   timestamp: 2021-01-27T04:22:54+00:00

from __future__ import annotations

from typing import Optional

from pydantic import BaseModel


class ServersInfoEntryModel(BaseModel):
    name: Optional[str] = None
    machine_identifier: Optional[str] = None
    host: Optional[str] = None
    port: Optional[str] = None
    version: Optional[str] = None



import json
import os
from typing import List


def __supported_api_versions__() -> List[str]:
    with open(os.path.join(os.path.dirname(__file__), "API_VERSIONS.json")) as f:
        versions = f.read()
        return json.loads(versions)

def __min_api_version__() -> str:
    return __supported_api_versions__()[0]



# generated by datamodel-codegen:
#   filename:  data.json
#   timestamp: 2022-08-26T22:43:25+00:00

from __future__ import annotations

from typing import Any, Dict, List, Optional, Union

from pydantic import BaseModel


class ConfigModel(BaseModel):
    incl_poster: Optional[int] = None
    html_support: Optional[int] = None
    chat_id: Optional[str] = None
    bot_token: Optional[str] = None
    incl_subject: Optional[int] = None
    disable_web_preview: Optional[int] = None


class ActionsModel(BaseModel):
    on_play: Optional[int] = None
    on_stop: Optional[int] = None


class OnPlayModel(BaseModel):
    subject: Optional[str] = None
    body: Optional[str] = None


class OnStopModel(BaseModel):
    subject: Optional[str] = None
    body: Optional[str] = None


class NotifyTextModel(BaseModel):
    on_play: Optional[OnPlayModel] = None
    on_stop: Optional[OnStopModel] = None


class NotifierConfigModel(BaseModel):
    id: Optional[int] = None
    agent_id: Optional[int] = None
    agent_name: Optional[str] = None
    agent_label: Optional[str] = None
    friendly_name: Optional[str] = None
    config: Optional[ConfigModel] = None
    config_options: Optional[List[Union[Dict[str, Any], str]]] = None
    actions: Optional[ActionsModel] = None
    notify_text: Optional[NotifyTextModel] = None






# generated by datamodel-codegen:
#   filename:  data.json
#   timestamp: 2021-01-27T04:23:37+00:00

from __future__ import annotations

from typing import Any, List, Optional

from pydantic import BaseModel


class IFTTTModel(BaseModel):
    ifttt_on_stop: Optional[bool] = None
    ifttt_on_pause: Optional[bool] = None
    ifttt_key: Optional[str] = None
    ifttt_on_concurrent: Optional[bool] = None
    ifttt_on_created: Optional[bool] = None
    ifttt_on_buffer: Optional[bool] = None
    ifttt_enabled: Optional[bool] = None
    ifttt_on_pmsupdate: Optional[bool] = None
    ifttt_on_newdevice: Optional[bool] = None
    ifttt_on_intdown: Optional[bool] = None
    ifttt_on_extup: Optional[bool] = None
    ifttt_on_intup: Optional[bool] = None
    ifttt_on_resume: Optional[bool] = None
    ifttt_on_watched: Optional[bool] = None
    ifttt_event: Optional[str] = None
    ifttt_on_extdown: Optional[bool] = None
    ifttt_on_play: Optional[bool] = None


class GrowlModel(BaseModel):
    growl_enabled: Optional[bool] = None
    growl_on_play: Optional[bool] = None
    growl_on_stop: Optional[bool] = None
    growl_on_buffer: Optional[bool] = None
    growl_on_pause: Optional[bool] = None
    growl_on_intdown: Optional[bool] = None
    growl_host: Optional[str] = None
    growl_on_extup: Optional[bool] = None
    growl_on_newdevice: Optional[bool] = None
    growl_on_pmsupdate: Optional[bool] = None
    growl_password: Optional[str] = None
    growl_on_resume: Optional[bool] = None
    growl_on_created: Optional[bool] = None
    growl_on_concurrent: Optional[bool] = None
    growl_on_watched: Optional[bool] = None
    growl_on_intup: Optional[bool] = None
    growl_on_extdown: Optional[bool] = None


class PMSModel(BaseModel):
    pms_use_bif: Optional[bool] = None
    pms_uuid: Optional[str] = None
    pms_url: Optional[str] = None
    pms_update_channel: Optional[str] = None
    pms_update_distro: Optional[str] = None
    pms_name: Optional[str] = None
    pms_logs_folder: Optional[str] = None
    pms_plexpass: Optional[int] = None
    pms_token: Optional[str] = None
    pms_port: Optional[int] = None
    pms_is_remote: Optional[int] = None
    pms_identifier: Optional[str] = None
    pms_ssl: Optional[int] = None
    pms_version: Optional[str] = None
    pms_update_distro_build: Optional[str] = None
    pms_ip: Optional[str] = None
    pms_logs_line_cap: Optional[int] = None
    pms_platform: Optional[str] = None
    pms_url_manual: Optional[int] = None
    pms_web_url: Optional[str] = None
    pms_is_cloud: Optional[int] = None
    pms_url_override: Optional[str] = None


class MonitoringModel(BaseModel):
    notify_upload_posters: Optional[int] = None
    movie_notify_enable: Optional[bool] = None
    notify_on_resume_body_text: Optional[str] = None
    notify_on_pmsupdate_body_text: Optional[str] = None
    refresh_libraries_on_startup: Optional[bool] = None
    session_db_write_attempts: Optional[int] = None
    notify_on_created_body_text: Optional[str] = None
    buffer_wait: Optional[int] = None
    notify_on_intup_subject_text: Optional[str] = None
    notify_recently_added_grandparent: Optional[bool] = None
    notify_on_newdevice_subject_text: Optional[str] = None
    notify_on_extup_subject_text: Optional[str] = None
    notify_on_watched_subject_text: Optional[str] = None
    notify_on_pause_subject_text: Optional[str] = None
    notify_on_extup_body_text: Optional[str] = None
    notify_on_watched_body_text: Optional[str] = None
    music_notify_on_pause: Optional[bool] = None
    notify_on_start_body_text: Optional[str] = None
    notify_on_pause_body_text: Optional[str] = None
    notify_on_buffer_body_text: Optional[str] = None
    music_notify_on_stop: Optional[bool] = None
    tv_notify_on_stop: Optional[bool] = None
    notify_on_stop_body_text: Optional[str] = None
    notify_on_created_subject_text: Optional[str] = None
    notify_on_extdown_body_text: Optional[str] = None
    notify_scripts_args_text: Optional[str] = None
    music_logging_enable: Optional[bool] = None
    movie_notify_on_stop: Optional[bool] = None
    notify_watched_percent: Optional[int] = None
    notify_on_concurrent_subject_text: Optional[str] = None
    notify_on_concurrent_body_text: Optional[str] = None
    tv_logging_enable: Optional[bool] = None
    tv_notify_enable: Optional[bool] = None
    notify_recently_added_delay: Optional[int] = None
    notify_on_intdown_body_text: Optional[str] = None
    logging_ignore_interval: Optional[int] = None
    notify_on_newdevice_body_text: Optional[str] = None
    refresh_users_interval: Optional[int] = None
    monitoring_use_websocket: Optional[bool] = None
    monitor_pms_updates: Optional[bool] = None
    notify_on_start_subject_text: Optional[str] = None
    notify_recently_added: Optional[bool] = None
    notify_on_pmsupdate_subject_text: Optional[str] = None
    music_notify_enable: Optional[bool] = None
    movie_notify_on_start: Optional[bool] = None
    notify_concurrent_by_ip: Optional[bool] = None
    refresh_users_on_startup: Optional[bool] = None
    buffer_threshold: Optional[int] = None
    notify_on_extdown_subject_text: Optional[str] = None
    refresh_libraries_interval: Optional[int] = None
    tv_notify_on_start: Optional[bool] = None
    video_logging_enable: Optional[bool] = None
    notify_on_resume_subject_text: Optional[str] = None
    notify_concurrent_threshold: Optional[int] = None
    notify_on_stop_subject_text: Optional[str] = None
    movie_notify_on_pause: Optional[bool] = None
    notify_consecutive: Optional[bool] = None
    movie_logging_enable: Optional[bool] = None
    notify_on_intup_body_text: Optional[str] = None
    notify_on_intdown_subject_text: Optional[str] = None
    music_notify_on_start: Optional[bool] = None
    tv_notify_on_pause: Optional[bool] = None
    monitor_remote_access: Optional[bool] = None
    imgur_client_id: Optional[str] = None
    monitoring_interval: Optional[int] = None
    notify_on_buffer_subject_text: Optional[str] = None
    tv_watched_percent: Optional[int] = None
    notify_group_recently_added_grandparent: Optional[bool] = None
    notify_recently_added_upgrade: Optional[bool] = None
    notify_group_recently_added: Optional[bool] = None
    notify_group_recently_added_parent: Optional[bool] = None
    movie_watched_percent: Optional[int] = None
    music_watched_percent: Optional[int] = None
    notify_continued_session_threshold: Optional[int] = None
    notify_remote_access_threshold: Optional[int] = None
    notify_new_device_initial_only: Optional[bool] = None


class GetFileSizesHoldModel(BaseModel):
    section_ids: Optional[List] = None
    rating_keys: Optional[List] = None


class GeneralModel(BaseModel):
    week_start_monday: Optional[bool] = None
    git_token: Optional[str] = None
    home_stats_cards: Optional[List[str]] = None
    backup_interval: Optional[int] = None
    graph_days: Optional[int] = None
    graph_type: Optional[str] = None
    get_file_sizes: Optional[bool] = None
    http_basic_auth: Optional[bool] = None
    api_sql: Optional[bool] = None
    http_root: Optional[str] = None
    http_proxy: Optional[bool] = None
    freeze_db: Optional[bool] = None
    ip_logging_enable: Optional[bool] = None
    launch_browser: Optional[bool] = None
    http_username: Optional[str] = None
    https_create_cert: Optional[bool] = None
    do_not_override_git_branch: Optional[bool] = None
    home_stats_length: Optional[str] = None
    http_host: Optional[str] = None
    first_run_complete: Optional[bool] = None
    enable_https: Optional[bool] = None
    check_github: Optional[bool] = None
    api_enabled: Optional[bool] = None
    git_path: Optional[str] = None
    update_labels: Optional[bool] = None
    config_version: Optional[str] = None
    check_github_interval: Optional[int] = None
    https_ip: Optional[str] = None
    group_history_tables: Optional[bool] = None
    https_domain: Optional[str] = None
    graph_tab: Optional[str] = None
    geoip_db: Optional[str] = None
    http_password: Optional[str] = None
    home_stats_count: Optional[str] = None
    backup_dir: Optional[str] = None
    git_user: Optional[str] = None
    interface: Optional[str] = None
    cache_images: Optional[bool] = None
    log_blacklist: Optional[bool] = None
    graph_months: Optional[str] = None
    cleanup_files: Optional[bool] = None
    check_github_on_startup: Optional[bool] = None
    http_port: Optional[int] = None
    log_dir: Optional[str] = None
    update_db_interval: Optional[int] = None
    allow_guest_access: Optional[bool] = None
    time_format: Optional[str] = None
    cache_dir: Optional[str] = None
    home_sections: Optional[List[str]] = None
    backup_days: Optional[int] = None
    https_cert: Optional[str] = None
    api_key: Optional[str] = None
    update_section_ids: Optional[bool] = None
    date_format: Optional[str] = None
    home_stats_type: Optional[bool] = None
    http_environment: Optional[str] = None
    history_table_activity: Optional[bool] = None
    https_key: Optional[str] = None
    http_hash_password: Optional[bool] = None
    home_library_cards: Optional[List[str]] = None
    http_hashed_password: Optional[bool] = None
    git_branch: Optional[str] = None
    anon_redirect: Optional[str] = None
    http_plex_admin: Optional[bool] = None
    update_show_changelog: Optional[int] = None
    show_advanced_settings: Optional[bool] = None
    git_remote: Optional[str] = None
    https_cert_chain: Optional[str] = None
    git_repo: Optional[str] = None
    home_stats_recently_added_count: Optional[str] = None
    themoviedb_apikey: Optional[str] = None
    tvmaze_lookup: Optional[bool] = None
    update_notifiers_db: Optional[bool] = None
    themoviedb_lookup: Optional[bool] = None
    plexpy_auto_update: Optional[bool] = None
    update_libraries_db_notify: Optional[bool] = None
    home_refresh_interval: Optional[int] = None
    http_base_url: Optional[str] = None
    win_sys_tray: Optional[bool] = None
    musicbrainz_lookup: Optional[bool] = None
    geoip_db_update_days: Optional[int] = None
    maxmind_license_key: Optional[str] = None
    geoip_db_installed: Optional[bool] = None
    launch_startup: Optional[bool] = None
    sys_tray_icon: Optional[bool] = None
    export_dir: Optional[str] = None
    http_rate_limit_attempts: Optional[int] = None
    http_rate_limit_attempts_interval: Optional[int] = None
    http_rate_limit_lockout_time: Optional[int] = None
    get_file_sizes_hold: Optional[GetFileSizesHoldModel] = None
    interface_list: Optional[List[str]] = None


class SlackModel(BaseModel):
    slack_on_stop: Optional[bool] = None
    slack_icon_emoji: Optional[str] = None
    slack_on_watched: Optional[bool] = None
    slack_on_extup: Optional[bool] = None
    slack_on_resume: Optional[bool] = None
    slack_hook: Optional[str] = None
    slack_incl_subject: Optional[bool] = None
    slack_on_pmsupdate: Optional[bool] = None
    slack_channel: Optional[str] = None
    slack_on_buffer: Optional[bool] = None
    slack_on_extdown: Optional[bool] = None
    slack_username: Optional[str] = None
    slack_incl_pmslink: Optional[bool] = None
    slack_enabled: Optional[bool] = None
    slack_incl_poster: Optional[bool] = None
    slack_on_pause: Optional[bool] = None
    slack_on_concurrent: Optional[bool] = None
    slack_on_intup: Optional[bool] = None
    slack_on_intdown: Optional[bool] = None
    slack_on_play: Optional[bool] = None
    slack_on_created: Optional[bool] = None
    slack_on_newdevice: Optional[bool] = None


class BoxcarModel(BaseModel):
    boxcar_sound: Optional[str] = None
    boxcar_on_newdevice: Optional[bool] = None
    boxcar_on_watched: Optional[bool] = None
    boxcar_on_buffer: Optional[bool] = None
    boxcar_on_stop: Optional[bool] = None
    boxcar_on_extdown: Optional[bool] = None
    boxcar_on_pmsupdate: Optional[bool] = None
    boxcar_on_intdown: Optional[bool] = None
    boxcar_on_pause: Optional[bool] = None
    boxcar_on_play: Optional[bool] = None
    boxcar_on_created: Optional[bool] = None
    boxcar_on_intup: Optional[bool] = None
    boxcar_on_resume: Optional[bool] = None
    boxcar_on_concurrent: Optional[bool] = None
    boxcar_enabled: Optional[bool] = None
    boxcar_on_extup: Optional[bool] = None
    boxcar_token: Optional[str] = None


class PushBulletModel(BaseModel):
    pushbullet_on_play: Optional[bool] = None
    pushbullet_channel_tag: Optional[str] = None
    pushbullet_on_buffer: Optional[bool] = None
    pushbullet_apikey: Optional[str] = None
    pushbullet_on_resume: Optional[bool] = None
    pushbullet_on_concurrent: Optional[bool] = None
    pushbullet_on_created: Optional[bool] = None
    pushbullet_on_pause: Optional[bool] = None
    pushbullet_on_extdown: Optional[bool] = None
    pushbullet_on_stop: Optional[bool] = None
    pushbullet_on_extup: Optional[bool] = None
    pushbullet_on_pmsupdate: Optional[bool] = None
    pushbullet_enabled: Optional[bool] = None
    pushbullet_on_watched: Optional[bool] = None
    pushbullet_on_intdown: Optional[bool] = None
    pushbullet_deviceid: Optional[str] = None
    pushbullet_on_intup: Optional[bool] = None
    pushbullet_on_newdevice: Optional[bool] = None


class ScriptsModel(BaseModel):
    scripts_on_newdevice: Optional[bool] = None
    scripts_on_stop: Optional[bool] = None
    scripts_on_intdown_script: Optional[str] = None
    scripts_on_newdevice_script: Optional[str] = None
    scripts_folder: Optional[str] = None
    scripts_on_pmsupdate: Optional[bool] = None
    scripts_on_buffer: Optional[bool] = None
    scripts_on_concurrent: Optional[bool] = None
    scripts_on_intup_script: Optional[str] = None
    scripts_on_intdown: Optional[bool] = None
    scripts_on_resume: Optional[bool] = None
    scripts_on_extdown_script: Optional[str] = None
    scripts_on_extup: Optional[bool] = None
    scripts_on_play_script: Optional[str] = None
    scripts_on_created_script: Optional[str] = None
    scripts_on_pmsupdate_script: Optional[str] = None
    scripts_on_resume_script: Optional[str] = None
    scripts_on_stop_script: Optional[str] = None
    scripts_on_watched_script: Optional[str] = None
    scripts_enabled: Optional[bool] = None
    scripts_on_extup_script: Optional[str] = None
    scripts_on_extdown: Optional[bool] = None
    scripts_on_created: Optional[bool] = None
    scripts_on_concurrent_script: Optional[str] = None
    scripts_on_pause_script: Optional[str] = None
    scripts_on_pause: Optional[bool] = None
    scripts_on_intup: Optional[bool] = None
    scripts_timeout: Optional[int] = None
    scripts_on_watched: Optional[bool] = None
    scripts_on_buffer_script: Optional[str] = None
    scripts_on_play: Optional[bool] = None


class NMAModel(BaseModel):
    nma_on_pmsupdate: Optional[bool] = None
    nma_priority: Optional[bool] = None
    nma_on_intdown: Optional[bool] = None
    nma_on_buffer: Optional[bool] = None
    nma_on_play: Optional[bool] = None
    nma_on_newdevice: Optional[bool] = None
    nma_on_resume: Optional[bool] = None
    nma_on_concurrent: Optional[bool] = None
    nma_on_watched: Optional[bool] = None
    nma_apikey: Optional[str] = None
    nma_on_intup: Optional[bool] = None
    nma_on_stop: Optional[bool] = None
    nma_on_created: Optional[bool] = None
    nma_on_pause: Optional[bool] = None
    nma_enabled: Optional[bool] = None
    nma_on_extdown: Optional[bool] = None
    nma_on_extup: Optional[bool] = None


class HipchatModel(BaseModel):
    hipchat_emoticon: Optional[str] = None
    hipchat_incl_subject: Optional[bool] = None
    hipchat_on_extdown: Optional[bool] = None
    hipchat_on_intup: Optional[bool] = None
    hipchat_enabled: Optional[bool] = None
    hipchat_on_created: Optional[bool] = None
    hipchat_on_newdevice: Optional[bool] = None
    hipchat_on_play: Optional[bool] = None
    hipchat_on_resume: Optional[bool] = None
    hipchat_on_buffer: Optional[bool] = None
    hipchat_incl_poster: Optional[bool] = None
    hipchat_on_watched: Optional[bool] = None
    hipchat_color: Optional[str] = None
    hipchat_on_concurrent: Optional[bool] = None
    hipchat_url: Optional[str] = None
    hipchat_on_pmsupdate: Optional[bool] = None
    hipchat_on_extup: Optional[bool] = None
    hipchat_incl_pmslink: Optional[bool] = None
    hipchat_on_pause: Optional[bool] = None
    hipchat_on_intdown: Optional[bool] = None
    hipchat_on_stop: Optional[bool] = None


class PushalotModel(BaseModel):
    pushalot_apikey: Optional[str] = None
    pushalot_on_pmsupdate: Optional[bool] = None
    pushalot_enabled: Optional[bool] = None
    pushalot_on_extdown: Optional[bool] = None
    pushalot_on_created: Optional[bool] = None
    pushalot_on_pause: Optional[bool] = None
    pushalot_on_stop: Optional[bool] = None
    pushalot_on_buffer: Optional[bool] = None
    pushalot_on_intup: Optional[bool] = None
    pushalot_on_newdevice: Optional[bool] = None
    pushalot_on_concurrent: Optional[bool] = None
    pushalot_on_watched: Optional[bool] = None
    pushalot_on_play: Optional[bool] = None
    pushalot_on_extup: Optional[bool] = None
    pushalot_on_intdown: Optional[bool] = None
    pushalot_on_resume: Optional[bool] = None


class ProwlModel(BaseModel):
    prowl_on_created: Optional[bool] = None
    prowl_on_play: Optional[bool] = None
    prowl_on_extdown: Optional[bool] = None
    prowl_on_pause: Optional[bool] = None
    prowl_on_intdown: Optional[bool] = None
    prowl_on_intup: Optional[bool] = None
    prowl_enabled: Optional[bool] = None
    prowl_priority: Optional[bool] = None
    prowl_on_pmsupdate: Optional[bool] = None
    prowl_on_newdevice: Optional[bool] = None
    prowl_on_extup: Optional[bool] = None
    prowl_on_watched: Optional[bool] = None
    prowl_on_buffer: Optional[bool] = None
    prowl_on_resume: Optional[bool] = None
    prowl_on_stop: Optional[bool] = None
    prowl_on_concurrent: Optional[bool] = None
    prowl_keys: Optional[str] = None


class AdvancedModel(BaseModel):
    verify_ssl_cert: Optional[bool] = None
    journal_mode: Optional[str] = None
    pms_timeout: Optional[int] = None
    cache_sizemb: Optional[int] = None
    session_db_write_attempts: Optional[int] = None
    system_analytics: Optional[bool] = None
    notification_threads: Optional[int] = None
    remote_access_ping_threshold: Optional[int] = None
    websocket_connection_timeout: Optional[int] = None
    config_version: Optional[int] = None
    metadata_cache_seconds: Optional[int] = None
    websocket_connection_attempts: Optional[int] = None
    jwt_secret: Optional[str] = None
    websocket_monitor_ping_pong: Optional[bool] = None
    synchronous_mode: Optional[str] = None
    pms_update_check_interval: Optional[int] = None
    jwt_update_secret: Optional[bool] = None
    verbose_logs: Optional[bool] = None
    add_live_tv_library: Optional[bool] = None
    remote_access_ping_interval: Optional[int] = None
    check_github_cache_seconds: Optional[int] = None
    export_threads: Optional[int] = None


class FacebookModel(BaseModel):
    facebook_app_secret: Optional[str] = None
    facebook_on_stop: Optional[bool] = None
    facebook_token: Optional[str] = None
    facebook_on_buffer: Optional[bool] = None
    facebook_on_play: Optional[bool] = None
    facebook_enabled: Optional[bool] = None
    facebook_on_intup: Optional[bool] = None
    facebook_incl_poster: Optional[bool] = None
    facebook_on_pmsupdate: Optional[bool] = None
    facebook_on_intdown: Optional[bool] = None
    facebook_on_extup: Optional[bool] = None
    facebook_on_resume: Optional[bool] = None
    facebook_on_concurrent: Optional[bool] = None
    facebook_on_extdown: Optional[bool] = None
    facebook_group: Optional[str] = None
    facebook_on_pause: Optional[bool] = None
    facebook_app_id: Optional[str] = None
    facebook_incl_pmslink: Optional[bool] = None
    facebook_on_newdevice: Optional[bool] = None
    facebook_on_watched: Optional[bool] = None
    facebook_redirect_uri: Optional[str] = None
    facebook_incl_subject: Optional[bool] = None
    facebook_on_created: Optional[bool] = None


class EmailModel(BaseModel):
    email_from_name: Optional[str] = None
    email_bcc: Optional[str] = None
    email_on_pause: Optional[bool] = None
    email_smtp_password: Optional[str] = None
    email_tls: Optional[bool] = None
    email_smtp_port: Optional[str] = None
    email_on_watched: Optional[bool] = None
    email_smtp_server: Optional[str] = None
    email_on_intdown: Optional[bool] = None
    email_on_concurrent: Optional[bool] = None
    email_on_intup: Optional[bool] = None
    email_enabled: Optional[bool] = None
    email_on_buffer: Optional[bool] = None
    email_on_resume: Optional[bool] = None
    email_smtp_user: Optional[str] = None
    email_cc: Optional[str] = None
    email_on_newdevice: Optional[bool] = None
    email_on_stop: Optional[bool] = None
    email_on_extdown: Optional[bool] = None
    email_html_support: Optional[bool] = None
    email_on_extup: Optional[bool] = None
    email_on_created: Optional[bool] = None
    email_on_pmsupdate: Optional[bool] = None
    email_to: Optional[str] = None
    email_from: Optional[str] = None
    email_on_play: Optional[bool] = None


class OSXNotifyModel(BaseModel):
    osx_notify_on_intdown: Optional[bool] = None
    osx_notify_on_buffer: Optional[bool] = None
    osx_notify_app: Optional[str] = None
    osx_notify_on_created: Optional[bool] = None
    osx_notify_on_pmsupdate: Optional[bool] = None
    osx_notify_on_stop: Optional[bool] = None
    osx_notify_enabled: Optional[bool] = None
    osx_notify_on_intup: Optional[bool] = None
    osx_notify_on_watched: Optional[bool] = None
    osx_notify_on_pause: Optional[bool] = None
    osx_notify_on_concurrent: Optional[bool] = None
    osx_notify_on_newdevice: Optional[bool] = None
    osx_notify_on_play: Optional[bool] = None
    osx_notify_on_extdown: Optional[bool] = None
    osx_notify_on_extup: Optional[bool] = None
    osx_notify_on_resume: Optional[bool] = None


class PlexWatchModel(BaseModel):
    grouping_user_history: Optional[bool] = None
    plexwatch_database: Optional[str] = None
    grouping_global_history: Optional[bool] = None
    grouping_charts: Optional[bool] = None


class PushoverModel(BaseModel):
    pushover_enabled: Optional[bool] = None
    pushover_on_extup: Optional[bool] = None
    pushover_on_intup: Optional[bool] = None
    pushover_on_buffer: Optional[bool] = None
    pushover_html_support: Optional[bool] = None
    pushover_keys: Optional[str] = None
    pushover_incl_url: Optional[bool] = None
    pushover_on_pause: Optional[bool] = None
    pushover_on_play: Optional[bool] = None
    pushover_on_stop: Optional[bool] = None
    pushover_apitoken: Optional[str] = None
    pushover_incl_pmslink: Optional[bool] = None
    pushover_on_pmsupdate: Optional[bool] = None
    pushover_on_extdown: Optional[bool] = None
    pushover_priority: Optional[bool] = None
    pushover_on_intdown: Optional[bool] = None
    pushover_on_newdevice: Optional[bool] = None
    pushover_sound: Optional[str] = None
    pushover_on_created: Optional[bool] = None
    pushover_on_resume: Optional[bool] = None
    pushover_on_concurrent: Optional[bool] = None
    pushover_on_watched: Optional[bool] = None


class TelegramModel(BaseModel):
    telegram_on_buffer: Optional[bool] = None
    telegram_bot_token: Optional[str] = None
    telegram_incl_poster: Optional[bool] = None
    telegram_on_resume: Optional[bool] = None
    telegram_on_pmsupdate: Optional[bool] = None
    telegram_html_support: Optional[bool] = None
    telegram_on_created: Optional[bool] = None
    telegram_incl_subject: Optional[bool] = None
    telegram_disable_web_preview: Optional[bool] = None
    telegram_enabled: Optional[bool] = None
    telegram_on_concurrent: Optional[bool] = None
    telegram_on_intdown: Optional[bool] = None
    telegram_on_extdown: Optional[bool] = None
    telegram_on_pause: Optional[bool] = None
    telegram_on_newdevice: Optional[bool] = None
    telegram_on_watched: Optional[bool] = None
    telegram_on_stop: Optional[bool] = None
    telegram_on_play: Optional[bool] = None
    telegram_on_extup: Optional[bool] = None
    telegram_chat_id: Optional[str] = None
    telegram_on_intup: Optional[bool] = None


class TwitterModel(BaseModel):
    twitter_on_resume: Optional[bool] = None
    twitter_on_pmsupdate: Optional[bool] = None
    twitter_on_buffer: Optional[bool] = None
    twitter_on_concurrent: Optional[bool] = None
    twitter_enabled: Optional[bool] = None
    twitter_on_watched: Optional[bool] = None
    twitter_consumer_secret: Optional[str] = None
    twitter_on_extup: Optional[bool] = None
    twitter_on_intdown: Optional[bool] = None
    twitter_on_intup: Optional[bool] = None
    twitter_access_token_secret: Optional[str] = None
    twitter_on_newdevice: Optional[bool] = None
    twitter_on_stop: Optional[bool] = None
    twitter_consumer_key: Optional[str] = None
    twitter_access_token: Optional[str] = None
    twitter_incl_subject: Optional[bool] = None
    twitter_on_extdown: Optional[bool] = None
    twitter_on_pause: Optional[bool] = None
    twitter_on_play: Optional[bool] = None
    twitter_incl_poster: Optional[bool] = None
    twitter_on_created: Optional[bool] = None


class PlexModel(BaseModel):
    plex_on_created: Optional[bool] = None
    plex_on_concurrent: Optional[bool] = None
    plex_client_host: Optional[str] = None
    plex_on_stop: Optional[bool] = None
    plex_on_resume: Optional[bool] = None
    plex_on_watched: Optional[bool] = None
    plex_on_play: Optional[bool] = None
    plex_on_extdown: Optional[bool] = None
    plex_on_intdown: Optional[bool] = None
    plex_on_newdevice: Optional[bool] = None
    plex_enabled: Optional[bool] = None
    plex_on_buffer: Optional[bool] = None
    plex_on_intup: Optional[bool] = None
    plex_on_pmsupdate: Optional[bool] = None
    plex_username: Optional[str] = None
    plex_on_pause: Optional[bool] = None
    plex_password: Optional[str] = None
    plex_on_extup: Optional[bool] = None


class JoinModel(BaseModel):
    join_on_concurrent: Optional[bool] = None
    join_on_newdevice: Optional[bool] = None
    join_deviceid: Optional[str] = None
    join_on_play: Optional[bool] = None
    join_on_watched: Optional[bool] = None
    join_on_resume: Optional[bool] = None
    join_on_created: Optional[bool] = None
    join_on_extdown: Optional[bool] = None
    join_on_intup: Optional[bool] = None
    join_on_buffer: Optional[bool] = None
    join_incl_subject: Optional[bool] = None
    join_enabled: Optional[bool] = None
    join_on_pause: Optional[bool] = None
    join_apikey: Optional[str] = None
    join_on_pmsupdate: Optional[bool] = None
    join_on_intdown: Optional[bool] = None
    join_on_extup: Optional[bool] = None
    join_on_stop: Optional[bool] = None


class XBMCModel(BaseModel):
    xbmc_password: Optional[str] = None
    xbmc_on_play: Optional[bool] = None
    xbmc_on_pause: Optional[bool] = None
    xbmc_username: Optional[str] = None
    xbmc_on_extup: Optional[bool] = None
    xbmc_on_watched: Optional[bool] = None
    xbmc_on_buffer: Optional[bool] = None
    xbmc_enabled: Optional[bool] = None
    xbmc_host: Optional[str] = None
    xbmc_on_created: Optional[bool] = None
    xbmc_on_intup: Optional[bool] = None
    xbmc_on_extdown: Optional[bool] = None
    xbmc_on_pmsupdate: Optional[bool] = None
    xbmc_on_stop: Optional[bool] = None
    xbmc_on_newdevice: Optional[bool] = None
    xbmc_on_resume: Optional[bool] = None
    xbmc_on_intdown: Optional[bool] = None
    xbmc_on_concurrent: Optional[bool] = None


class BrowserModel(BaseModel):
    browser_on_intdown: Optional[bool] = None
    browser_on_newdevice: Optional[bool] = None
    browser_on_pmsupdate: Optional[bool] = None
    browser_on_intup: Optional[bool] = None
    browser_on_buffer: Optional[bool] = None
    browser_auto_hide_delay: Optional[int] = None
    browser_on_created: Optional[bool] = None
    browser_on_pause: Optional[bool] = None
    browser_on_stop: Optional[bool] = None
    browser_on_play: Optional[bool] = None
    browser_on_watched: Optional[bool] = None
    browser_on_extdown: Optional[bool] = None
    browser_on_extup: Optional[bool] = None
    browser_on_resume: Optional[bool] = None
    browser_on_concurrent: Optional[bool] = None
    browser_enabled: Optional[bool] = None


class NewsletterModel(BaseModel):
    newsletter_dir: Optional[str] = None
    newsletter_templates: Optional[str] = None
    newsletter_self_hosted: Optional[bool] = None
    newsletter_static_url: Optional[bool] = None
    newsletter_inline_styles: Optional[bool] = None
    newsletter_password: Optional[str] = None
    newsletter_custom_dir: Optional[str] = None
    newsletter_auth: Optional[bool] = None


class CloudinaryModel(BaseModel):
    cloudinary_api_key: Optional[str] = None
    cloudinary_cloud_name: Optional[str] = None
    cloudinary_api_secret: Optional[str] = None


class SettingsModel(BaseModel):
    IFTTT: Optional[IFTTTModel] = None
    Growl: Optional[GrowlModel] = None
    PMS: Optional[PMSModel] = None
    Monitoring: Optional[MonitoringModel] = None
    General: Optional[GeneralModel] = None
    Slack: Optional[SlackModel] = None
    Boxcar: Optional[BoxcarModel] = None
    PushBullet: Optional[PushBulletModel] = None
    Scripts: Optional[ScriptsModel] = None
    NMA: Optional[NMAModel] = None
    Hipchat: Optional[HipchatModel] = None
    Pushalot: Optional[PushalotModel] = None
    Prowl: Optional[ProwlModel] = None
    Advanced: Optional[AdvancedModel] = None
    Facebook: Optional[FacebookModel] = None
    Email: Optional[EmailModel] = None
    OSX_Notify: Optional[OSXNotifyModel] = None
    PlexWatch: Optional[PlexWatchModel] = None
    Pushover: Optional[PushoverModel] = None
    Telegram: Optional[TelegramModel] = None
    Twitter: Optional[TwitterModel] = None
    Plex: Optional[PlexModel] = None
    Join: Optional[JoinModel] = None
    XBMC: Optional[XBMCModel] = None
    Browser: Optional[BrowserModel] = None
    Newsletter: Optional[NewsletterModel] = None
    Cloudinary: Optional[CloudinaryModel] = None



# generated by datamodel-codegen:
#   filename:  data.json
#   timestamp: 2021-01-27T04:34:08+00:00

from __future__ import annotations

from typing import Any, Optional, Union

from pydantic import BaseModel


class GeoIPLookupModel(BaseModel):
    city: Optional[str] = None
    code: Optional[str] = None
    continent: Optional[str] = None
    country: Optional[str] = None
    latitude: Optional[Union[float, None]] = None
    longitude: Optional[Union[float, None]] = None
    postal_code: Optional[str] = None
    region: Optional[str] = None
    timezone: Optional[str] = None
    accuracy: Optional[Any] = None






# generated by datamodel-codegen:
#   filename:  data.json
#   timestamp: 2021-01-27T04:21:25+00:00

from __future__ import annotations

from typing import Any, Optional

from pydantic import BaseModel


class ServerInfoModel(BaseModel):
    pms_name: Optional[str] = None
    pms_version: Optional[str] = None
    pms_platform: Optional[str] = None
    pms_ip: Optional[str] = None
    pms_port: Optional[int] = None
    pms_ssl: Optional[int] = None
    pms_is_remote: Optional[int] = None
    pms_url: Optional[str] = None
    pms_url_manual: Optional[int] = None
    pms_identifier: Optional[str] = None
    pms_plexpass: Optional[int] = None






# generated by datamodel-codegen:
#   filename:  data.json
#   timestamp: 2021-01-27T04:59:53+00:00

from __future__ import annotations

from typing import Any, List, Optional

from pydantic import BaseModel


class UserModel(BaseModel):
    row_id: Optional[int] = None
    user_id: Optional[int] = None
    username: Optional[str] = None
    friendly_name: Optional[str] = None
    user_thumb: Optional[str] = None
    email: Optional[str] = None
    is_active: Optional[int] = None
    is_admin: Optional[int] = None
    is_home_user: Optional[int] = None
    is_allow_sync: Optional[int] = None
    is_restricted: Optional[int] = None
    do_notify: Optional[int] = None
    keep_history: Optional[int] = None
    allow_guest: Optional[int] = None
    deleted_user: Optional[int] = None
    shared_libraries: Optional[List] = None






from datetime import datetime

from tautulli.internal.utils import (
    datetime_to_string
)

def test_datetime_to_string():
    date = datetime(2021, 1, 1, 0, 0, 0)

    # default format: "%Y-%m-%d"
    assert datetime_to_string(datetime_object=date) == "2021-01-01"

    fmt = "%Y-%m-%d %H:%M:%S"
    assert datetime_to_string(datetime_object=date, string_format=fmt) == "2021-01-01 00:00:00"



from typing import Union, List

import objectrest

from tautulli.internal import static
from tautulli.models.activity import build_summary_from_activity_json
from tautulli.tools.utils import url_encode
from tautulli.exceptions import PlexException

from plexapi.server import PlexServer


# noinspection PyTypeChecker
class APIShortcuts:
    def __init__(self, api: "RawAPI"):
        self._api = api
        self._plex_api = None

    def __init_subclass__(cls, **kwargs):
        super().__init_subclass__(**kwargs)

    def ping(self) -> bool:
        """
        Ping the Tautulli server

        :returns: `True` if successful, `False` if unsuccessful
        :rtype: bool
        """
        name = self._api.server_friendly_name
        return name is not None and name is not static.empty_string

    def ping_plex(self, pms_url: str = None) -> bool:
        """
        Ping the Plex Media Server
        :param pms_url: URL of the Plex Media Server, defaults to default PMS URL
        :type pms_url: str, optional
        :returns: `True` if successful, `False` if unsuccessful
        :rtype: bool
        """
        pms_url = pms_url or self._api.server_info.get('pms_url', None)

        if pms_url is None:
            return False
        try:
            status_code = objectrest.get(url=pms_url, verify_ssl=False).status_code
            return 200 <= status_code < 300 or status_code == 401  # 401 is a valid response
        except Exception:
            return False

    @property
    def has_plex_pass(self) -> bool:
        """
        Check if the Plex Media Server has Plex Pass

        :returns: `True` if successful, `False` if unsuccessful
        :rtype: bool
        """
        return self._api.server_info.get('pms_plexpass', 0) == 1

    @property
    def api_version(self) -> str:
        """
        Get the Tautulli API version

        :returns: API version
        :rtype: str
        """
        return self._api.tautulli_info.get('tautulli_version', None)

    @property
    def plex_api(self) -> Union[PlexServer, None]:
        """
        Get a configured Plex API connection to the Plex Media Server configured in Tautulli.

        This **may** require running a server-side SQL query to retrieve the admin user's Plex Media Server token.
        Enable SQL queries by setting `api_sql = 1` in Tautulli's config.ini file while the application is not running.

        :returns: Plex API object, or None if it could not be configured
        :rtype: plexapi.server.PlexServer or None
        """
        if self._plex_api is None:
            server_url: Union[str, None] = self._api.server_info.get('pms_url', None)
            server_token: Union[str, None] = self._get_plex_server_token()

            if not server_url or not server_token:
                return None

            try:
                self._plex_api: PlexServer = PlexServer(baseurl=server_url, token=server_token)
            except Exception as e:
                raise PlexException(f"Could not connect to the Plex Media Server: {e}")

        return self._plex_api

    def _get_plex_server_token(self) -> Union[str, None]:
        """
        Get the Plex Media Server token used by the Tautulli instance

        :returns: Plex Media Server token
        :rtype: str
        """
        # Attempt to get it from the settings
        pms_settings: Union[dict, None] = self._api.get_settings(key='PMS')
        if pms_settings:
            pms_token: Union[str, None] = pms_settings.get('pms_token', None)
            if pms_token:
                return pms_token

        # Attempt to get it via an admin user in the database (there should only be one)
        sql_query: str = "SELECT server_token FROM users WHERE is_admin == 1 LIMIT 1"
        sql_response: Union[List[dict], None] = self._api.sql(query=sql_query)
        if sql_response:
            first_entry: dict = sql_response[0]
            return first_entry.get('server_token', None)

        return None

    def get_library_by_name(self, library_name: str) -> dict:
        """
        Get a Plex Media Server library using its name

        :returns: Dict of data
        :rtype: dict
        """
        for library in self._api.library_names:
            if library.get('section_name') == library_name:
                return library
        return {}

    @property
    def activity_summary(self) -> dict:
        """
        Get a summary of current activity on the Plex Media Server

        :returns: Dict of data
        :rtype: dict
        """
        _activity_data = self._api.activity()
        return build_summary_from_activity_json(activity_data=_activity_data).model_dump()

    @property
    def activity_summary_message(self) -> str:
        """
        Get a summary message of current activity on the Plex Media Server

        :returns: Activity summary message
        :rtype: str
        """
        _activity_data = self._api.activity()
        # Yes, this is the JSON API using an object as a middleman.
        return build_summary_from_activity_json(activity_data=_activity_data).message

    def get_plex_image_url_from_proxy(self, plex_image_path: str, image_type: str = 'poster') -> str:
        """
        Get a URL to a Plex Media Server image hosted by the Tautulli image proxy

        :param plex_image_path: Path to the image on the Plex Media Server
        :type plex_image_path: str
        :param image_type: Image type, defaults to 'poster'
        :type image_type: str, optional
        :return: The URL to the image
        :rtype: str
        """
        return f"{self._api._base_url}/pms_image_proxy?img={url_encode(plex_image_path)}&fallback={image_type}"

    def download_plex_image_from_proxy(self, plex_image_path: str, file_path: str, file_format: str = 'png',
                                       image_type: str = 'poster') -> None:
        """
        Download an image from the Plex Media Server using the Tautulli API and save it to a local file

        :param plex_image_path: Path to the image on the Plex Media Server
        :type plex_image_path: str
        :param file_path: Path to the file to save the image to
        :type file_path: str
        :param file_format: Image file format, defaults to 'png'
        :type file_format: str, optional
        :param image_type: Image type, defaults to 'poster'
        :type image_type: str, optional
        :returns: None
        """
        data = self._api.pms_image_proxy(img=plex_image_path, fallback=image_type, img_format=file_format)

        with open(file_path, 'wb') as file:
            file.write(data)

    def get_link_to_open_media_item_in_browser(self, media_item_id: int) -> str:
        """
        Get a link to open a media item in the Plex Web app

        :param media_item_id: ID of the media item
        :type media_item_id: int
        :returns: URL to open the media item in the Plex Web app
        :rtype: str
        """
        plex_server_details = self._api.server_info
        pms_url = plex_server_details.get('pms_url')
        pms_id = plex_server_details.get('pms_identifier')

        if not pms_url or not pms_id:
            return None

        key = url_encode(f"/library/metadata/{media_item_id}")

        return f"{pms_url}#!/server/{pms_id}/details?key={key}"



from typing import get_type_hints, List


def create_attributes(obj: object):
    for attribute, type_hint in get_type_hints(obj).items():
        setattr(obj, attribute, type_hint)

def make_object(json_data: dict, obj: object):
    create_attributes(obj)
    for key, value in json_data.items():
        attribute = getattr(obj, key)
        if attribute:
            # if attribute is a basic type
            if obj.__annotations__[key] in [str, bytes, int, float, bool]:
                setattr(obj, key, value)
                continue
            # if attribute is a list
            try:
                if obj.__annotations__[key].__origin__ == list:
                    base_type = obj.__annotations__[key].__args__[0]
                    temp_list = [base_type(item) for item in value]
                    setattr(obj, key, temp_list)
                    continue
            except:
                pass
            # if attribute is a dict
            if obj.__annotations__[key] == dict:
                pass
                continue
            # if attribute is a specific class
            else:
                attribute_type = obj.__annotations__[key]
                setattr(obj, key, attribute_type(value))


# generated by datamodel-codegen:
#   filename:  data.json
#   timestamp: 2021-01-27T01:23:27+00:00

from __future__ import annotations

from typing import Any, List, Optional

from pydantic import BaseModel

from tautulli.internal import utils as internal_utils, static
from tautulli.tools.utils import to_human_bitrate


class SessionModel(BaseModel):
    session_key: Optional[str] = None
    media_type: Optional[str] = None
    view_offset: Optional[str] = None
    progress_percent: Optional[str] = None
    quality_profile: Optional[str] = None
    synced_version_profile: Optional[str] = None
    optimized_version_profile: Optional[str] = None
    user: Optional[str] = None
    channel_stream: Optional[int] = None
    section_id: Optional[str] = None
    library_name: Optional[str] = None
    rating_key: Optional[str] = None
    parent_rating_key: Optional[str] = None
    grandparent_rating_key: Optional[str] = None
    title: Optional[str] = None
    parent_title: Optional[str] = None
    grandparent_title: Optional[str] = None
    original_title: Optional[str] = None
    sort_title: Optional[str] = None
    media_index: Optional[str] = None
    parent_media_index: Optional[str] = None
    studio: Optional[str] = None
    content_rating: Optional[str] = None
    summary: Optional[str] = None
    tagline: Optional[str] = None
    rating: Optional[str] = None
    rating_image: Optional[str] = None
    audience_rating: Optional[str] = None
    audience_rating_image: Optional[str] = None
    user_rating: Optional[str] = None
    duration: Optional[str] = None
    year: Optional[str] = None
    thumb: Optional[str] = None
    parent_thumb: Optional[str] = None
    grandparent_thumb: Optional[str] = None
    art: Optional[str] = None
    banner: Optional[str] = None
    originally_available_at: Optional[str] = None
    added_at: Optional[str] = None
    updated_at: Optional[str] = None
    last_viewed_at: Optional[str] = None
    guid: Optional[str] = None
    parent_guid: Optional[str] = None
    grandparent_guid: Optional[str] = None
    directors: Optional[List] = None
    writers: Optional[List] = None
    actors: Optional[List] = None
    genres: Optional[List[str]] = None
    labels: Optional[List] = None
    collections: Optional[List] = None
    guids: Optional[List] = None
    full_title: Optional[str] = None
    children_count: Optional[int] = None
    live: Optional[int] = None
    id: Optional[str] = None
    container: Optional[str] = None
    bitrate: Optional[str] = None
    height: Optional[str] = None
    width: Optional[str] = None
    aspect_ratio: Optional[str] = None
    video_codec: Optional[str] = None
    video_resolution: Optional[str] = None
    video_full_resolution: Optional[str] = None
    video_framerate: Optional[str] = None
    video_profile: Optional[str] = None
    audio_codec: Optional[str] = None
    audio_channels: Optional[str] = None
    audio_channel_layout: Optional[str] = None
    audio_profile: Optional[str] = None
    optimized_version: Optional[int] = None
    channel_call_sign: Optional[str] = None
    channel_id: Optional[str] = None
    channel_identifier: Optional[str] = None
    channel_title: Optional[str] = None
    channel_thumb: Optional[str] = None
    channel_vcn: Optional[str] = None
    file: Optional[str] = None
    file_size: Optional[str] = None
    indexes: Optional[int] = None
    selected: Optional[int] = None
    type: Optional[str] = None
    video_codec_level: Optional[str] = None
    video_bitrate: Optional[str] = None
    video_bit_depth: Optional[str] = None
    video_chroma_subsampling: Optional[str] = None
    video_color_primaries: Optional[str] = None
    video_color_range: Optional[str] = None
    video_color_space: Optional[str] = None
    video_color_trc: Optional[str] = None
    video_frame_rate: Optional[str] = None
    video_ref_frames: Optional[str] = None
    video_height: Optional[str] = None
    video_width: Optional[str] = None
    video_language: Optional[str] = None
    video_language_code: Optional[str] = None
    video_scan_type: Optional[str] = None
    audio_bitrate: Optional[str] = None
    audio_bitrate_mode: Optional[str] = None
    audio_sample_rate: Optional[str] = None
    audio_language: Optional[str] = None
    audio_language_code: Optional[str] = None
    subtitle_codec: Optional[str] = None
    subtitle_container: Optional[str] = None
    subtitle_format: Optional[str] = None
    subtitle_forced: Optional[int] = None
    subtitle_location: Optional[str] = None
    subtitle_language: Optional[str] = None
    subtitle_language_code: Optional[str] = None
    row_id: Optional[int] = None
    user_id: Optional[int] = None
    username: Optional[str] = None
    friendly_name: Optional[str] = None
    user_thumb: Optional[str] = None
    email: Optional[str] = None
    is_active: Optional[int] = None
    is_admin: Optional[int] = None
    is_home_user: Optional[int] = None
    is_allow_sync: Optional[int] = None
    is_restricted: Optional[int] = None
    do_notify: Optional[int] = None
    keep_history: Optional[int] = None
    deleted_user: Optional[int] = None
    allow_guest: Optional[int] = None
    shared_libraries: Optional[List[str]] = None
    ip_address: Optional[str] = None
    ip_address_public: Optional[str] = None
    device: Optional[str] = None
    platform: Optional[str] = None
    platform_name: Optional[str] = None
    platform_version: Optional[str] = None
    product: Optional[str] = None
    product_version: Optional[str] = None
    profile: Optional[str] = None
    player: Optional[str] = None
    machine_id: Optional[str] = None
    state: Optional[str] = None
    local: Optional[int] = None
    relayed: Optional[int] = None
    secure: Optional[int] = None
    session_id: Optional[str] = None
    bandwidth: Optional[str] = None
    location: Optional[str] = None
    transcode_key: Optional[str] = None
    transcode_throttled: Optional[int] = None
    transcode_progress: Optional[int] = None
    transcode_speed: Optional[str] = None
    transcode_audio_channels: Optional[str] = None
    transcode_audio_codec: Optional[str] = None
    transcode_video_codec: Optional[str] = None
    transcode_width: Optional[str] = None
    transcode_height: Optional[str] = None
    transcode_container: Optional[str] = None
    transcode_protocol: Optional[str] = None
    transcode_hw_requested: Optional[int] = None
    transcode_hw_decode: Optional[str] = None
    transcode_hw_decode_title: Optional[str] = None
    transcode_hw_encode: Optional[str] = None
    transcode_hw_encode_title: Optional[str] = None
    transcode_hw_full_pipeline: Optional[int] = None
    transcode_max_offset_available: Optional[int] = None
    transcode_min_offset_available: Optional[int] = None
    audio_decision: Optional[str] = None
    video_decision: Optional[str] = None
    subtitle_decision: Optional[str] = None
    throttled: Optional[str] = None
    transcode_hw_decoding: Optional[int] = None
    transcode_hw_encoding: Optional[int] = None
    stream_container: Optional[str] = None
    stream_bitrate: Optional[str] = None
    stream_aspect_ratio: Optional[str] = None
    stream_audio_codec: Optional[str] = None
    stream_audio_channels: Optional[str] = None
    stream_audio_channel_layout: Optional[str] = None
    stream_video_codec: Optional[str] = None
    stream_video_framerate: Optional[str] = None
    stream_video_resolution: Optional[str] = None
    stream_video_height: Optional[str] = None
    stream_video_width: Optional[str] = None
    stream_duration: Optional[str] = None
    stream_container_decision: Optional[str] = None
    optimized_version_title: Optional[str] = None
    synced_version: Optional[int] = None
    live_uuid: Optional[str] = None
    bif_thumb: Optional[str] = None
    subtitles: Optional[int] = None
    transcode_decision: Optional[str] = None
    container_decision: Optional[str] = None
    stream_video_full_resolution: Optional[str] = None
    video_dynamic_range: Optional[str] = None
    stream_video_dynamic_range: Optional[str] = None
    stream_video_bitrate: Optional[str] = None
    stream_video_bit_depth: Optional[str] = None
    stream_video_chroma_subsampling: Optional[str] = None
    stream_video_color_primaries: Optional[str] = None
    stream_video_color_range: Optional[str] = None
    stream_video_color_space: Optional[str] = None
    stream_video_color_trc: Optional[str] = None
    stream_video_codec_level: Optional[str] = None
    stream_video_ref_frames: Optional[str] = None
    stream_video_language: Optional[str] = None
    stream_video_language_code: Optional[str] = None
    stream_video_scan_type: Optional[str] = None
    stream_video_decision: Optional[str] = None
    stream_audio_bitrate: Optional[str] = None
    stream_audio_bitrate_mode: Optional[str] = None
    stream_audio_sample_rate: Optional[str] = None
    stream_audio_channel_layout_: Optional[str] = None
    stream_audio_language: Optional[str] = None
    stream_audio_language_code: Optional[str] = None
    stream_audio_decision: Optional[str] = None
    stream_subtitle_codec: Optional[str] = None
    stream_subtitle_container: Optional[str] = None
    stream_subtitle_format: Optional[str] = None
    stream_subtitle_forced: Optional[int] = None
    stream_subtitle_location: Optional[str] = None
    stream_subtitle_language: Optional[str] = None
    stream_subtitle_language_code: Optional[str] = None
    stream_subtitle_decision: Optional[str] = None
    stream_subtitle_transient: Optional[int] = None

    @property
    def duration_milliseconds(self):
        try:
            return int(self.duration)
        except:
            return 0

    @property
    def location_milliseconds(self):
        try:
            return int(self.view_offset)
        except:
            return 0

    @property
    def progress_percentage(self):
        if not self.duration_milliseconds:
            return 0
        return int(self.location_milliseconds / self.duration_milliseconds)

    @property
    def progress_marker(self):
        if not self.location_milliseconds or not self.duration_milliseconds:
            return ""
        current_progress_min_sec = internal_utils.milliseconds_to_minutes_seconds(
            milliseconds=self.location_milliseconds)
        total_min_sec = internal_utils.milliseconds_to_minutes_seconds(milliseconds=self.duration_milliseconds)
        return f"{current_progress_min_sec}/{total_min_sec}"

    @property
    def eta(self):
        if not self.duration_milliseconds or not self.location_milliseconds:
            return ""
        milliseconds_remaining = self.duration_milliseconds - self.location_milliseconds
        eta_datetime = internal_utils.now_plus_milliseconds(milliseconds=milliseconds_remaining)
        eta_string = internal_utils.datetime_to_string(datetime_object=eta_datetime, string_format="%H:%M")
        return eta_string

    @property
    def status_icon(self):
        """
        Get icon for a stream state

        :returns: emoji icon
        """
        return static.switcher.get(self.state, "")

    @property
    def type_icon(self):
        if self.media_type in static.media_type_icons:
            return static.media_type_icons[self.media_type]
        # thanks twilsonco
        elif self.live:
            return static.media_type_icons['live']
        return ""

    @property
    def human_bandwidth(self) -> str:
        try:
            return to_human_bitrate(float(self.bandwidth))
        except:
            return to_human_bitrate(0)

    @property
    def transcoding_stub(self):
        return ' (Transcode)' if self.stream_container_decision == 'transcode' else ''

    @property
    def summary(self) -> str:
        return f"{static.session_title_message.format(icon=self.status_icon, username=self.username, media_type_icon=self.type_icon, title=self.title)}\n" \
               f"{static.session_player_message.format(product=self.product, player=self.player)}\n" \
               f"{static.session_details_message.format(quality_profile=self.quality_profile, bandwidth=self.human_bandwidth, transcoding=self.transcoding_stub)}\n" \
               f"{static.session_progress_message.format(progress=self.progress_marker, eta=self.eta)}"


class ActivityModel(BaseModel):
    stream_count: Optional[str] = None
    sessions: Optional[List[SessionModel]] = None
    stream_count_direct_play: Optional[int] = None
    stream_count_direct_stream: Optional[int] = None
    stream_count_transcode: Optional[int] = None
    total_bandwidth: Optional[int] = None
    lan_bandwidth: Optional[int] = None
    wan_bandwidth: Optional[int] = None

    @property
    def summary(self) -> ActivitySummaryModel:
        return build_summary_from_activity_object(activity=self)


class ActivitySummaryModel(BaseModel):
    stream_count: Optional[str] = "0"
    transcode_count: Optional[int] = 0
    total_bandwidth: Optional[int] = 0
    lan_bandwidth: Optional[int] = 0

    @property
    def message(self):
        """
        Get activity summary message

        :returns: Summary message
        :rtype: Optional[str]
        """
        overview_message = ""
        stream_count = 0
        try:
            stream_count = int(self.stream_count)
        except:
            pass

        if stream_count > 0:
            overview_message += static.sessions_message.format(stream_count=stream_count,
                                                               word=internal_utils.make_plural(word='stream',
                                                                                               count=stream_count))
            if self.transcode_count > 0:
                overview_message += f" ({static.transcodes_message.format(transcode_count=self.transcode_count, word=internal_utils.make_plural(word='transcode', count=self.transcode_count))}) "

        if self.total_bandwidth > 0:
            overview_message += f" | {static.bandwidth_message.format(bandwidth=to_human_bitrate(float(self.total_bandwidth)))}"
            if self.lan_bandwidth > 0:
                overview_message += f" {static.lan_bandwidth_message.format(bandwidth=to_human_bitrate(float(self.lan_bandwidth)))}"

        return overview_message


def build_summary_from_activity_object(activity: ActivityModel) -> ActivitySummaryModel:
    """
    Create an ActivitySummary using an Activity object

    :param activity: Activity object to use for Overview
    :type activity: Activity
    :returns: ActivitySummaryModel object
    :rtype: ActivitySummaryModel
    """
    overview = ActivitySummaryModel()
    overview.stream_count = activity.stream_count
    overview.transcode_count = activity.stream_count_transcode
    overview.total_bandwidth = activity.total_bandwidth
    overview.lan_bandwidth = activity.lan_bandwidth

    return overview


def build_summary_from_activity_json(activity_data: dict) -> ActivitySummaryModel:
    """
    Create an ActivitySummary using Activity JSON data

    :param activity_data: Activity JSON data to use for Overview
    :type activity_data: dict
    :returns: ActivitySummaryModel object
    :rtype: ActivitySummaryModel
    """
    return ActivitySummaryModel(**activity_data)



# generated by datamodel-codegen:
#   filename:  data.json
#   timestamp: 2021-01-27T04:38:48+00:00

from __future__ import annotations

from typing import Any, List, Optional

from pydantic import BaseModel


class LibraryUserStatsModel(BaseModel):
    friendly_name: Optional[str] = None
    user_id: Optional[int] = None
    user_thumb: Optional[str] = None
    username: Optional[str] = None
    total_plays: Optional[int] = None






# generated by datamodel-codegen:
#   filename:  data.json
#   timestamp: 2021-01-27T03:52:33+00:00

from __future__ import annotations

from typing import Any, Optional

from pydantic import BaseModel


class NewsletterNotificationModel(BaseModel):
    newsletter_notification_id: Optional[int] = None






# generated by datamodel-codegen:
#   filename:  data.json
#   timestamp: 2021-01-27T03:54:30+00:00

from __future__ import annotations

from typing import Any, List, Optional

from pydantic import BaseModel


class NotifierParameterModel(BaseModel):
    name: Optional[str] = None
    type: Optional[str] = None
    value: Optional[str] = None



# generated by datamodel-codegen:
#   filename:  data.json
#   timestamp: 2021-01-27T04:45:21+00:00

from __future__ import annotations

from typing import Any, List, Optional

from pydantic import BaseModel


class DatumModel(BaseModel):
    id: Optional[int] = None
    timestamp: Optional[int] = None
    newsletter_id: Optional[int] = None
    agent_id: Optional[int] = None
    agent_name: Optional[str] = None
    notify_action: Optional[str] = None
    subject_text: Optional[str] = None
    body_text: Optional[str] = None
    start_date: Optional[str] = None
    end_date: Optional[str] = None
    uuid: Optional[str] = None
    success: Optional[int] = None


class NewsletterLogModel(BaseModel):
    recordsFiltered: Optional[int] = None
    recordsTotal: Optional[int] = None
    data: Optional[List[DatumModel]] = None
    draw: Optional[int] = None






# generated by datamodel-codegen:
#   filename:  data.json
#   timestamp: 2022-08-26T20:59:57+00:00

from __future__ import annotations

from typing import Any, List, Optional

from pydantic import BaseModel


class SeriesModel(BaseModel):
    name: Optional[str] = None
    data: Optional[List] = None


class PlaysOrStreamTypesByModel(BaseModel):
    categories: Optional[List[str]] = None
    series: Optional[List[SeriesModel]] = None






# generated by datamodel-codegen:
#   filename:  data.json
#   timestamp: 2022-08-26T22:23:14+00:00

from __future__ import annotations

from typing import Any, List, Optional

from pydantic import BaseModel


class DatumModel(BaseModel):
    timestamp: Optional[int] = None
    art_level: Optional[int] = None
    complete: Optional[int] = None
    custom_fields: str
    exists: Optional[bool] = None
    export_id: Optional[int] = None
    exported_items: Optional[int] = None
    file_format: Optional[str] = None
    file_size: Optional[int] = None
    filename: Optional[Any] = None
    individual_files: Optional[int] = None
    media_info_level: Optional[int] = None
    media_type: Optional[str] = None
    media_type_title: Optional[str] = None
    metadata_level: Optional[int] = None
    rating_key: Optional[Any] = None
    section_id: Optional[int] = None
    thumb_level: Optional[int] = None
    title: Optional[str] = None
    total_items: Optional[int] = None
    user_id: Optional[Any] = None


class ExportsTableModel(BaseModel):
    draw: Optional[int] = None
    recordsTotal: Optional[int] = None
    recordsFiltered: Optional[int] = None
    data: Optional[List[DatumModel]] = None






# generated by datamodel-codegen:
#   filename:  data.json
#   timestamp: 2021-01-27T05:05:19+00:00

from __future__ import annotations

from typing import Any, List, Optional

from pydantic import BaseModel


class UserWatchTimeStatsModel(BaseModel):
    query_days: Optional[int] = None
    total_plays: Optional[int] = None
    total_time: Optional[int] = None






# generated by datamodel-codegen:
#   filename:  data.json
#   timestamp: 2021-01-27T04:26:04+00:00

from __future__ import annotations

from typing import Optional

from pydantic import BaseModel


class UpdateCheckModel(BaseModel):
    update: Optional[bool] = None
    install_type: Optional[str] = None



# generated by datamodel-codegen:
#   filename:  data.json
#   timestamp: 2021-01-27T03:57:17+00:00

from __future__ import annotations

from typing import Any, Optional

from pydantic import BaseModel


class PMSUpdateModel(BaseModel):
    update_available: Optional[bool] = None
    platform: Optional[str] = None
    release_date: Optional[int] = None
    version: Optional[str] = None
    requirements: Optional[str] = None
    extra_info: Optional[str] = None
    changelog_added: Optional[str] = None
    changelog_fixed: Optional[str] = None
    label: Optional[str] = None
    distro: Optional[str] = None
    distro_build: Optional[str] = None
    download_url: Optional[str] = None



# generated by datamodel-codegen:
#   filename:  data.json
#   timestamp: 2021-01-27T04:35:02+00:00

from __future__ import annotations

from typing import Any, List, Union, Optional

from pydantic import BaseModel


class DatumModel(BaseModel):
    reference_id: Optional[int] = None
    row_id: Optional[int] = None
    id: Optional[int] = None
    date: Optional[int] = None
    started: Optional[int] = None
    stopped: Optional[int] = None
    duration: Optional[int] = None # Keep for backwards compatibility
    play_duration: Optional[int] = None
    paused_counter: Optional[int] = None
    user_id: Optional[int] = None
    user: Optional[str] = None
    friendly_name: Optional[str] = None
    user_thumb: Optional[str] = None
    platform: Optional[str] = None
    product: Optional[str] = None
    player: Optional[str] = None
    ip_address: Optional[str] = None
    live: Optional[int] = None
    machine_id: Optional[str] = None
    media_type: Optional[str] = None
    rating_key: Optional[int] = None
    parent_rating_key: Optional[Union[int, str]] = None
    grandparent_rating_key: Optional[Union[int, str]] = None
    full_title: Optional[str] = None
    title: Optional[str] = None
    parent_title: Optional[str] = None
    grandparent_title: Optional[str] = None
    original_title: Optional[str] = None
    year: Optional[Union[int, str]] = None
    media_index: Optional[Union[int, str]] = None
    parent_media_index: Optional[Union[int, str]] = None
    thumb: Optional[str] = None
    originally_available_at: Optional[str] = None
    guid: Optional[str] = None
    transcode_decision: Optional[str] = None
    percent_complete: Optional[int] = None
    watched_status: Optional[float] = None
    group_count: Optional[int] = None
    group_ids: Optional[str] = None
    state: Optional[Any] = None
    session_key: Optional[Any] = None


class HistoryModel(BaseModel):
    recordsFiltered: Optional[int] = None
    recordsTotal: Optional[int] = None
    data: Optional[List[DatumModel]] = None
    draw: Optional[int] = None
    filter_duration: Optional[str] = None
    total_duration: Optional[str] = None






# generated by datamodel-codegen:
#   filename:  data.json
#   timestamp: 2021-01-27T04:56:39+00:00

from __future__ import annotations

from typing import Any, List, Optional

from pydantic import BaseModel


class SyncedItemsModel(BaseModel):
    audio_bitrate: Optional[str] = None
    client_id: Optional[str] = None
    content_type: Optional[str] = None
    device_name: Optional[str] = None
    failure: Optional[str] = None
    item_complete_count: Optional[str] = None
    item_count: Optional[str] = None
    item_downloaded_count: Optional[str] = None
    item_downloaded_percent_complete: Optional[int] = None
    metadata_type: Optional[str] = None
    photo_quality: Optional[str] = None
    platform: Optional[str] = None
    rating_key: Optional[str] = None
    root_title: Optional[str] = None
    state: Optional[str] = None
    sync_id: Optional[str] = None
    sync_title: Optional[str] = None
    total_size: Optional[str] = None
    user: Optional[str] = None
    user_id: Optional[str] = None
    username: Optional[str] = None
    video_bitrate: Optional[str] = None
    video_quality: Optional[str] = None



